# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy.types import Enum
from sqlalchemy import (Column,
                        Integer,
                        String,
                        DateTime,
                        ForeignKey,
                        Boolean,
                        Float,
                        )

from carepoint.models.state import EnumOrderState


class Order(Carepoint.BASE):
    __tablename__ = 'CsOm'
    __dbname__ = 'cph'

    order_id = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    acct_id = Column(Integer)
    invoice_nbr = Column(Integer)
    order_state_cn = Column(
        Integer,
        ForeignKey('CsOmStatus.state_cn')
    )
    order_status_cn = Column(
        Integer,
        ForeignKey('CsOmStatus.OmStatus'),
    )
    hold_yn = Column(Boolean)
    priority_cn = Column(Integer)
    submit_date = Column(DateTime)
    fax_sent = Column(DateTime)
    fax_returned = Column(DateTime)
    ship_text = Column(String)
    rph_id = Column(Integer)
    tech_fill = Column(String)
    rph_check = Column(String)
    comments = Column(String)
    liCount = Column(Integer)
    c2Count = Column(Integer)
    ship_cn = Column(Integer)
    sat_deliv_yn = Column(Boolean)
    ship_amt = Column(Float)
    store_id = Column(
        Integer,
        ForeignKey('csstore.store_id'),
    )
    ship_flag_cn = Column(Integer)
    Locked_yn = Column(Boolean)
    locked_id = Column(Integer)
    lock_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    ship_date = Column(DateTime)
    expected_by = Column(DateTime)
    owner_store_id = Column(
        Integer,
        ForeignKey('csstore.store_id'),
    )
    auto_charge_cc_id = Column(Integer)
    auto_charge_cc_state_cn = Column(Integer)
    send_pos_state_cn = Column(Integer)
    automated_yn = Column(Boolean)
    status_cn = Column(Integer)
    order_category_cn = Column(Integer)
    route_store_id = Column(
        Integer,
        ForeignKey('csstore.store_id'),
    )
    DestStateCD = Column(String)
    add_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    add_date = Column(DateTime)
    chg_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    chg_date = Column(DateTime)

    @property
    def order_state(self):
        return EnumOrderState(self.order_state_cn)



# -*- coding: utf-8 -*-
# Copyright 2016-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import imp
import operator
import os
import urllib2

from contextlib import contextmanager

from sqlalchemy import bindparam, or_
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.inspection import inspect
from sqlalchemy.orm import sessionmaker
from sqlalchemy import text


from .db import Db
from smb.SMBHandler import SMBHandler


Base = declarative_base()
Base.get = lambda s, k, v=None: getattr(s, k, v)
Base.__getitem__ = lambda s, k, v=None: getattr(s, k, v)
Base.__setitem__ = lambda s, k, v: setattr(s, k, v)


models, env, dbs = {}, {}, {}


class Carepoint(dict):
    """ Base CarePoint db connector object """

    BASE = Base
    DEFAULT_DB = 'cph'

    # Default path to search for models - change with register_model_dir
    model_path = os.path.join(os.path.dirname(__file__), '..', 'models')

    FILTERS = {
        '>=': operator.ge,
        '>': operator.gt,
        '<=': operator.le,
        '<': operator.lt,
        '=': operator.eq,
        '==': operator.eq,
    }

    def __init__(
        self, server, user, passwd, smb_user=None, smb_passwd=None,
        db_args=None, **engine_args
    ):
        """ It initializes new Carepoint object

        Args:
            server (str): IP or Hostname to database
            user (str): Username for database
            passwd (str): Password for database
            smb_user (str): Username to use for SMB connection, ``None`` to
                use the database user
            smd_passwd (str): Password to use for the SMB connection, ``None``
                to use the database password
            db_args (dict): Dictionary of arguments to send during initial
                db creation
            **engine_args (mixed): Kwargs to pass to ``create_engine``
        """

        super(Carepoint, self).__init__()
        global dbs
        self.env = {}
        self.dbs = dbs
        self.iter_refresh = False
        params = {
            'user': user,
            'passwd': passwd,
            'server': server,
            'db': 'cph',
        }
        if db_args is not None:
            params.update(db_args)
        if engine_args:
            params.update(engine_args)
        if smb_user is None:
            self.smb_creds = {
                'user': user,
                'passwd': passwd,
            }
        else:
            self.smb_creds = {
                'user': smb_user,
                'passwd': smb_passwd,
            }
        self.db_params = params
        self._init_env(False)

    def _init_env(self, clear=False):
        """ It initializes the global db and environments

        Params:
            clear: (bool) True to clear the global session
        """
        if clear:
            self.dbs.clear()
        # @TODO: Lazy load, once other dbs needed
        if not self.dbs.get('cph'):
            self.dbs['cph'] = Db(**self.db_params)
        if not self.env.get('cph'):
            self.env['cph'] = sessionmaker(
                autocommit=False,
                autoflush=False,
                bind=self.dbs['cph'],
                expire_on_commit=True,
            )

    def _get_model_session(self, model_obj):
        """ It yields a session for the model_obj """
        return self._get_session(model_obj.__dbname__)

    @contextmanager
    def _get_session(self, db_name):
        session = self.env[db_name]()
        try:
            yield session
            session.commit()
        except:
            session.rollback()
            raise
        finally:
            session.close()

    @property
    def _smb_prefix(self):
        """ Return URI prefix for SMB share """
        return 'smb://{user}:{passwd}@'.format(**self.smb_creds)

    def get_file(self, path):
        """ Return a file-like object for the SMB path

        Args:
            path: :type:`str` SMB path to fetch

        Returns:
            :type:`file` File interface object representing remote resource
        """
        opener = urllib2.build_opener(SMBHandler)
        return opener.open('%s%s' % (self._smb_prefix, path))

    def send_file(self, path, file_obj):
        """ Send a file-like object to the SMB path

        Args:
            path: :type:`str` SMB path to fetch
            file_obj: :type:`file` File interface object to send to server

        Returns:
            :type:`bool` Success
        """
        with urllib2.build_opener(SMBHandler) as opener:
            opener.open('%s%s' % (self._smb_prefix, path), data=file_obj)
        return True

    def _create_criterion(self, model_obj, col_name, operator, query):
        """ Create a SQLAlchemy criterion from filter parts

        Args:
            model_obj: :class:`sqlalchemy.Table` Table class to search
            col_name: :type:`str` Name of column to query
            operator: :type:`str` Domain operator to use in query
            query: :type:`str` Text to search for

        Returns:
            SQLAlchemy criterion representing a single WHERE clause

        Raises:
            NotImplementedError: When query operator is not implemented
            AttributeError: When col_name does not exist in the model_obj
        """

        try:
            col_obj = getattr(model_obj, col_name)
            operator_obj = self.FILTERS[operator]
            return operator_obj(col_obj, query)

        except KeyError:
            raise

        except AttributeError:
            raise

    def _unwrap_filters(self, model_obj, filters=None):
        """ Unwrap a dictionary of filters into something usable by SQLAlchemy
        :param model_obj: Table class to search
        :type model_obj: :class:`sqlalchemy.Table`
        :param filters: Filters, keyed by col name
        :type filters: dict
        :rtype: list
        """

        if filters is None:
            filters = {}

        new_filters = []
        for col_name, col_filter in filters.items():

            if isinstance(col_filter, dict):
                for _operator, _filter in col_filter.items():
                    new_filters.append(self._create_criterion(
                        model_obj, col_name, _operator, _filter
                    ))

            elif isinstance(col_filter, (list, tuple)):
                query = []
                for _filter in col_filter:
                    query.append(
                        self._create_criterion(
                            model_obj, col_name, '==', _filter,
                        ),
                    )
                new_filters.append(or_(*query))

            else:
                new_filters.append(self._create_criterion(
                    model_obj, col_name, '==', col_filter
                ))

        return new_filters

    def _create_entities(self, model_obj, cols):
        """ Return list of entities matching cols
        :param model_obj: Table class to search
        :type model_obj: :class:`sqlalchemy.Table`
        :param cols: List of col names
        :type cols: list
        :rtype: :type:`list` of :class:`sqlalchemy.Column`
        """
        out = []
        for col in cols:
            try:
                out.append(getattr(model_obj, col))
            except AttributeError:
                pass
        return out

    def read(self, model_obj, record_id, with_entities=None):
        """ Get record by id and return the object
        :param model_obj: Table class to search
        :type model_obj: :class:`sqlalchemy.Table`
        :param record_id: Id of record to manipulate
        :param with_entities: Attributes to rcv from db. None for *
        :type with_entities: list or None
        :param with_entities: List of col names to select, None for all
        :type with_entities: list or None
        :rtype: :class:`sqlalchemy.engine.ResultProxy`
        """
        with self._get_model_session(model_obj) as session:
            res = session.query(model_obj).get(record_id)
            if with_entities:
                res.with_entities(*self._create_entities(
                    model_obj, with_entities
                ))
            return res

    def search(self, model_obj, filters=None, with_entities=None):
        """ Search table by filters and return records
        :param model_obj: Table class to search
        :type model_obj: :class:`sqlalchemy.schema.Table`
        :param filters: Filters to apply to search
        :type filters: dict or None
        :param with_entities: List of col names to select, None for all
        :type with_entities: list or None
        :rtype: :class:`sqlalchemy.engine.ResultProxy`
        """
        with self._get_model_session(model_obj) as session:
            if filters is None:
                filters = {}
            filters = self._unwrap_filters(model_obj, filters)
            res =  session.query(model_obj).filter(*filters)
            if with_entities:
                res.with_entities(*self._create_entities(
                    model_obj, with_entities
                ))
            return res

    def create(self, model_obj, vals):
        """ Wrapper to create a record in Carepoint
        :param model_obj: Table class to create with
        :type model_obj: :class:`sqlalchemy.schema.Table`
        :param vals: Data to create record with
        :type vals: dict
        :rtype: :class:`sqlalchemy.ext.declarative.Declarative`
        """
        with self._get_model_session(model_obj) as session:
            record = model_obj(**vals)
            session.add(record)
            return record

    def update(self, model_obj, record_id, vals):
        """ Wrapper to update a record in Carepoint
        :param model_obj: Table class to update
        :type model_obj: :class:`sqlalchemy.schema.Table`
        :param record_id: Id of record to manipulate
        :type record_id: int
        :param vals: Data to create record with
        :type vals: dict
        :rtype: :class:`sqlalchemy.ext.declarative.Declarative`
        """
        with self._get_model_session(model_obj) as session:
            record = self.read(model_obj, record_id)
            for key, val in vals.items():
                setattr(record, key, val)
            return record

    def delete(self, model_obj, record_id):
        """ Wrapper to delete a record in Carepoint
        :param model_obj: Table class to update
        :type model_obj: :class:`sqlalchemy.schema.Table`
        :param record_id: Id of record to manipulate
        :type record_id: int
        :return: Whether the record was found, and deleted
        :rtype: bool
        """
        with self._get_model_session(model_obj) as session:
            record = self.read(model_obj, record_id)
            result_cnt = record.count()
            if result_cnt == 0:
                return False
            assert result_cnt == 1
            session.delete(record)
        return True

    def get_pks(self, model_obj):
        """ Return the Primary keys in the model
        :param model_obj: Table class to update
        :type model_obj: :class:`sqlalchemy.schema.Table`
        :return: Tuple of primary key name strings
        :rtype: tuple
        """
        return tuple(k.name for k in inspect(model_obj).primary_key)

    def get_next_sequence(self, sequence_name, db_name='cph'):
        """ It generates and returns the next int in sequence
        Params:
            sequence_name: ``str`` Name of the sequence in Carepoint DB
            db_name: ``str`` Name of DB containing sequence stored proc
        Return:
            Integer to use as pk
        """
        with self._get_session(db_name) as session:  
            res = session.connection().execute(
                text(
                    "SET NOCOUNT ON;"
                    "DECLARE @out int = 0;"
                    "EXEC CsGenerateIntId :seq_name, @out output;"
                    "SELECT @out;"
                    "SET NOCOUNT OFF;",
                    bindparams=[bindparam('seq_name')],
                ),
                seq_name=sequence_name,
            )
            id_int = res.fetchall()[0][0]
            return id_int

    def __getattr__(self, key):
        """ Re-implement __getattr__ to use __getitem__ if attr not found """
        try:
            return super(Carepoint, self).__getattr__(key)
        except AttributeError:
            try:
                self.__getitem__(key)
            except KeyError:
                raise AttributeError()

    def __setitem__(self, key, val, __global=False, *args, **kwargs):
        """ Re-implement __setitem__ to allow for global model sync """
        super(Carepoint, self).__setitem__(key, val, *args, **kwargs)
        if not __global:
            global models
            models[key] = val

    def __getitem__(self, key, retry=True, default=False):
        """ Re-implement __getitem__ to scan for models if key missing  """
        global models
        for k, v in models.iteritems():
            self.__setitem__(k, v, True)
        try:
            return super(Carepoint, self).__getitem__(key)
        except KeyError:
            if default is not False:
                return default
            elif retry:
                self.find_models()
                return self.__getitem__(key, False)
            else:
                raise KeyError(
                    'Plugin "%s" not found in model_dir "%s"' % (
                        key, self.model_path
                    )
                )

    def set_iter_refresh(self, refresh=True):
        """ Toggle flag to search for new models before iteration
        :param refresh: Whether to refresh before iteration
        :type refresh: bool
        """
        self.iter_refresh = refresh

    def __refresh_models__(self):
        if self.iter_refresh:
            self.find_models()

    def __iter__(self):
        """ Reimplement __iter__ to allow for optional model refresh """
        self.__refresh_models__()
        return super(Carepoint, self).__iter__()

    def values(self):
        """ Reimplement values to allow for optional model refresh """
        self.__refresh_models__()
        return super(Carepoint, self).values()

    def keys(self):
        """ Reimplement keys to allow for optional model refresh """
        self.__refresh_models__()
        return super(Carepoint, self).keys()

    def items(self):
        """ Reimplement items to allow for optional model refresh """
        self.__refresh_models__()
        return super(Carepoint, self).items()

    def itervalues(self):
        """ Reimplement itervalues to allow for optional model refresh """
        self.__refresh_models__()
        return super(Carepoint, self).itervalues()

    def iterkeys(self):
        """ Reimplement iterkeys to allow for optional model refresh """
        self.__refresh_models__()
        return super(Carepoint, self).iterkeys()

    def iteritems(self):
        """ Reimplement iteritems to allow for optional model refresh """
        self.__refresh_models__()
        return super(Carepoint, self).iteritems()

    def register_model(self, model_obj):
        """ Registration logic + append to models struct
        :param model_obj: Model object to register
        :type model_obj: :class:`sqlalchemy.ext.declarative.Declarative`
        """
        self[model_obj.__name__] = model_obj

    def register_model_dir(self, model_path):
        """ This function sets the model path to be searched
        :param model_path: Path of models
        :type model_path: str
        """
        if os.path.isdir(model_path):
            self.model_path = model_path
        else:
            raise EnvironmentError('%s is not a directory' % model_path)

    def find_models(self, model_path=None):
        """ Traverse registered model directory and import non-loaded modules
        """

        if model_path is None:
            model_path = self.model_path

        if model_path is not None and not os.path.isdir(model_path):
            raise EnvironmentError('%s is not a directory' % model_path)

        for dir_name, subdirs, files in os.walk(model_path):

            if dir_name.startswith('__'):
                continue
            dir_name = os.path.abspath(dir_name)
            parent_module = dir_name.replace(model_path, '')
            parent_module = parent_module.replace(os.path.sep, '.')

            for file_ in files:
                if file_.endswith('.py') and file_ != '__init__.py':
                    module = file_[:-3]
                    mod_obj = globals().get(module)
                    if mod_obj is None:
                        f, filename, desc = imp.find_module(
                            module, [dir_name]
                        )
                        mod_obj = imp.load_module(
                            module, f, filename, desc
                        )
                        cls = [
                            m for m in dir(mod_obj) if not m.startswith('__')
                        ]
                        for model_cls in cls:
                            model_obj = getattr(mod_obj, model_cls)
                            if hasattr(model_obj, '__tablename__'):
                                if not hasattr(model_obj, '__dbname__'):
                                    model_obj.__dbname__ = self.DEFAULT_DB
                                self.register_model(model_obj)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from mixer.backend.sqlalchemy import mixer
from datetime import datetime


dt_now = datetime.now()
__model__ = 'carepoint.models.cph.address.Address'


address_default = mixer.blend(
    __model__,
    addr_id=1,
    parent_addr_id=None,
    inherited_yn=False,
    owner_type_cn=1,
    alt_name='Alt Name',
    addr1='Addr1',
    addr2='Addr2',
    addr3='Addr3',
    city='City',
    state_cd='StateCd',
    zip='Zip',
    zip_plus4='ZipPlus4',
    country_cd='CountryCd',
    mailing_yn=False,
    anote='ANote',
    app_flags=1,
    timestmp=dt_now,
    add_user_id=1,
    add_date=dt_now,
    chd_user_id=1,
    chg_date=dt_now,
)


def addresses_rnd(cnt):
    return mixer.cycle(cnt).blend(__model__)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        String,
                        DateTime,
                        Boolean,
                        ForeignKey,
                        Numeric,
                        Text,
                        )


class Prescription(Carepoint.BASE):
    __tablename__ = 'cprx'
    __dbname__ = 'cph'

    rx_id = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    script_no = Column(String)
    old_script_no = Column(String)
    pat_id = Column(
        Integer,
        ForeignKey('cppat.pat_id'),
    )
    store_id = Column(
        Integer,
        ForeignKey('csstore.store_id'),
    )
    md_id = Column(
        Integer,
        ForeignKey('cpmd.md_id'),
    )
    ndc = Column(String)
    gcn_seqno = Column(Integer)
    mfg = Column(String)
    drug_name = Column(String)
    input_src_cn = Column(Integer)
    src_pat_meds_cn = Column(Integer)
    start_date = Column(DateTime)
    expire_date = Column(DateTime)
    stop_date = Column(DateTime)
    sig_code = Column(String)
    sig_text = Column(String)
    written_qty = Column(Numeric)
    starter_qty = Column(Numeric)
    days_supply = Column(Numeric)
    pkg_size = Column(Numeric)
    units_per_dose = Column(Numeric)
    units_entered = Column(String)
    awp = Column(Numeric)
    udef = Column(Numeric)
    ful = Column(Numeric)
    mac = Column(Numeric)
    aac = Column(Numeric)
    freq_entered = Column(String)
    freq_of_admin = Column(Numeric)
    sched_of_admin_cn = Column(Integer)
    daw_yn = Column(Boolean)
    refills_orig = Column(Numeric)
    refills_left = Column(Numeric)
    last_rxdisp_id = Column(
        Integer,
        ForeignKey('cprx_disp.rxdisp_id'),
    )
    last_refill_qty = Column(Numeric)
    last_refill_date = Column(DateTime)
    refill_date = Column(DateTime)
    src_org_id = Column(
        Integer,
        ForeignKey('csorg.org_id'),
    )
    cmt = Column(Text)
    exit_state_cn = Column(Integer)
    script_status_cn = Column(Integer)
    sched_date = Column(DateTime)
    sched_time = Column(DateTime)
    scheduled_yn = Column(Boolean)
    drug_dea_class = Column(String)
    manual_add_yn = Column(Boolean)
    status_cn = Column(Integer)
    life_date = Column(DateTime)
    self_prescribed_yn = Column(Boolean)
    last_transfer_type_io = Column(String)
    last_disp_prod = Column(String)
    transfer_cnt = Column(Numeric)
    wc_claim_id = Column(String)
    injury_date = Column(DateTime)
    gpi_rx = Column(Integer)
    auth_by = Column(String)
    orig_disp_date = Column(DateTime)
    short_term_yn = Column(Integer)
    orig_date = Column(DateTime)
    refills_used = Column(Numeric)
    wc_emp_id = Column(Integer)
    dose_unit = Column(String)
    dosage_multiplier = Column(Numeric)
    df = Column(String)
    uu = Column(String)
    app_flags = Column(Integer)
    timestmp = Column(DateTime)
    sched2_no = Column(String)
    orig_expire_date = Column(DateTime)
    rxq_status_cn = Column(Integer)
    IVRCmt = Column(String)
    wc_bodypart = Column(String)
    comp_ndc = Column(String)
    treatment_yn = Column(Boolean)
    ivr_callback = Column(String)
    autofill_yn = Column(Boolean)
    autofill_resume_date = Column(DateTime)
    workflow_status_cn = Column(Integer)
    extern_process_cn = Column(Integer)
    md_fac_id = Column(Integer)
    owner_store_id = Column(Integer)
    study_id = Column(Integer)
    min_days_until_refill = Column(Integer)
    sig_text_english = Column(String)
    order_fullfillment_cn = Column(Integer)
    taxable = Column(Integer)
    MAR_flag = Column(Integer)
    FreeFormStrength = Column(String)
    priority_cn = Column(Integer)
    ecs_yn = Column(Boolean)
    edit_locked_yn = Column(Boolean)
    locked_yn = Column(Boolean)
    locked_id = Column(Integer)
    locked_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    daw_rx_cn = Column(Integer)
    add_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    add_date = Column(DateTime)
    chg_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    chg_date = Column(DateTime)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        DateTime,
                        Numeric,
                        ForeignKey,
                        )


class FdbImgDate(Carepoint.BASE):
    __tablename__ = 'fdbrimguij'
    __dbname__ = 'cph'

    IMGUNIQID = Column(
        Numeric(10, 0),
        primary_key=True,
    )
    IMGSTRTDT = Column(
        DateTime,
        primary_key=True,
    )
    IMGSTOPDT = Column(
        DateTime,
    )
    IMGID = Column(
        Numeric(10, 0),
        ForeignKey('fdbrimgimg.IMGID'),
    )



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        String,
                        Integer,
                        Boolean,
                        )


class FdbAllergenGroup(Carepoint.BASE):
    __tablename__ = 'fdrdamagd'
    __dbname__ = 'cph'

    dam_agcsp = Column(
        Integer,
        primary_key=True,
    )
    dam_agcspd = Column(
        String(50),
    )
    potentially_inactive = Column(
        Boolean,
    )
    dam_agcsp_status = Column(
        Integer,
    )



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        Numeric,
                        ForeignKey,
                        DateTime,
                        String,
                        )


class AccountActivity(Carepoint.BASE):
    __tablename__ = 'CsSArActivity'
    __dbname__ = 'cph'

    id = Column(
        Integer,
        primary_key=True,
        autoincrement=False,
    )
    acct_id = Column(
        Integer,
        ForeignKey('cp_acct.ID'),
    )
    order_id = Column(
        Integer,
        ForeignKey('CsOm.order_id'),
    )
    pat_id = Column(
        Integer,
        ForeignKey('cppat.pat_id'),
    )
    rxdisp_id = Column(
        Integer,
        ForeignKey('cprx_disp.rxdisp_id'),
    )
    org_id = Column(
        Integer,
        ForeignKey('csorg.org_id'),
    )
    orig_org_id = Column(
        Integer,
        ForeignKey('csorg.org_id'),
    )
    item_id = Column(
        Integer,
        ForeignKey('item.item_id'),
    )
    descr = Column(String)
    refno = Column(String)
    ct_id = Column(Integer)
    code_num = Column(Integer)
    amt = Column(Numeric)
    reason_cn = Column(Integer)
    rpt_cat = Column(Integer)
    qty = Column(Numeric)
    tax = Column(Numeric)
    nonrx_order_line_id = Column(Integer)
    store_id = Column(
        Integer,
        ForeignKey('csstore.store_id'),
    )
    batch_id = Column(Integer)
    batch_no = Column(String)
    add_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    add_date = Column(DateTime)
    chg_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    chg_date = Column(DateTime)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import unittest
from sqlalchemy.schema import Table
from carepoint.tests.db.db import DatabaseTest
from carepoint.models.cph.order_status import OrderStatus, EnumOrderState


class TestModelsCphOrderStatus(DatabaseTest):

    def test_table_initialization(self, ):
        self.assertIsInstance(OrderStatus.__table__, Table)

    def new_record(self):
        self.type_cn = 20
        obj = OrderStatus()
        obj.state_cn = self.type_cn
        return obj

    def test_state(self):
        """ It should return proper Enum for state cn """
        obj = self.new_record()
        self.assertEqual(
            EnumOrderState(self.type_cn),
            obj.state,
        )


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).


class TestModel(object):
    __tablename__ = '__test__'
    __dbname__ = 'cph'
    test_col = 'Test'

    @classmethod
    def run(self):
        return True



# -*- coding: utf-8 -*-
# Copyright 2016-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from sqlalchemy import create_engine
from urllib import quote_plus as urlquote


class Db(object):
    ''' Base db connector object '''

    ODBC_DRIVER = 'FreeTDS&TDS_VERSION=8.0'
    SQLITE = 'sqlite'

    def __new__(
        self, server=None, user=None, passwd=None, db=None, port=1433,
        drv=ODBC_DRIVER, **engine_args
    ):
        """ It establishes a new database connection and returns engine

        Args:
            server (str): IP or Hostname to database
            user (str): Username for database
            passwd (str): Password for database
            db (str): Name of database
            port (int): Connection port
            drv (str): Name of underlying database driver for connection
            **engine_args (mixed): Kwargs to pass to ``create_engine``

        Return:
            sqlalchemy.engine.Engine
        """

        if drv != self.SQLITE:
            params = {
                'usr': urlquote(user),
                'pass': urlquote(passwd),
                'srv': server,
                'drv': drv,
                'db': db,
                'prt': port,
            }
            dsn = 'mssql+pyodbc://{usr}:{pass}@{srv}:{prt}/{db}?driver={drv}'
            return create_engine(dsn.format(**params), **engine_args)

        else:
            return create_engine('%s://' % self.SQLITE, **engine_args)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        Boolean,
                        String,
                        )


class FdbPemMoe(Carepoint.BASE):
    """ Monographs - Text Portion """
    __tablename__ = 'fdrpemmoe'
    __dbname__ = 'cph'

    pemono = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    pemono_sn = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    pemtxtei = Column(String)
    pemtxte = Column(String)
    pemgndr = Column(String)
    pemage = Column(String)
    update_yn = Column(Boolean)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        String,
                        DateTime,
                        Boolean,
                        ForeignKey,
                        Text,
                        )


class Doctor(Carepoint.BASE):
    __tablename__ = 'cpmd'
    __dbname__ = 'cph'

    md_id = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    lname = Column(String)
    lname_sdx = Column(String)
    fname = Column(String)
    fname_sdx = Column(String)
    mname = Column(String)
    title_lu = Column(String)
    suffix_lu = Column(String)
    degree_lu = Column(String)
    prescriber_type_cn = Column(Integer)
    email = Column(String)
    url = Column(String)
    pref_meth_contact_cn = Column(Integer)
    best_cont_time = Column(String)
    specialty_cd = Column(String)
    dea_no = Column(String)
    dea_suffix = Column(String)
    spin_no = Column(String)
    upin_no = Column(String)
    other_id = Column(String)
    fed_tax_id = Column(String)
    hin_id = Column(String)
    champus_id = Column(String)
    state_lic_id = Column(String)
    ncpdp_id = Column(String)
    medicaid_id = Column(String)
    medicare_id = Column(String)
    blue_shield_id = Column(String)
    blue_cross_id = Column(String)
    npi_id = Column(String)
    practice_org_id = Column(
        Integer,
        ForeignKey('csorg.org_id'),
    )
    cmt = Column(Text)
    status_cn = Column(Integer)
    conv_code = Column(String)
    app_flags = Column(Integer)
    timestmp = Column(DateTime)
    state_issued_id = Column(String)
    sname = Column(String)
    medicaid_restricted_yn = Column(Boolean)
    fac_id = Column(Integer)
    C2Restricted_yn = Column(Boolean)
    co_md_id = Column(
        Integer,
        ForeignKey('cpmd.md_id'),
    )
    do_not_refer = Column(Integer)
    deceased_yn = Column(Boolean)
    C0restricted_yn = Column(Boolean)
    C3restricted_yn = Column(Boolean)
    C4restricted_yn = Column(Boolean)
    C5restricted_yn = Column(Boolean)
    study_drugs_yn = Column(Boolean)
    tracking_cn = Column(Integer)
    CTP_no = Column(String)
    add_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    add_date = Column(DateTime)
    chg_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    chg_date = Column(DateTime)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        String,
                        Text,
                        DateTime,
                        ForeignKey,
                        )


class PatientAllergy(Carepoint.BASE):
    __tablename__ = 'cppat_alr'
    __dbname__ = 'cph'

    ptalr_id = Column(
        Integer, 
        primary_key=True,
    )
    pat_id = Column(
        Integer,
        ForeignKey('cppat.pat_id'),
    )
    name = Column(
        String(60),
    )
    hicl_seqno = Column(
        Integer,
        ForeignKey('fdrhiclsq.hicl_seqno'),
    )
    hic = Column(
        String(6),
        ForeignKey('fdrhicd.hic'),
    )
    dam_agcsp = Column(
        Integer,
        ForeignKey('fdrdamagd.dam_agcsp'),
    )
    alr_type_cn = Column(
        Integer,
    )
    onset_date = Column(
        DateTime,
    )
    resolution_status_cn = Column(
        Integer,
    )
    resolution_date = Column(
        DateTime,
    )
    screen_yn = Column(
        Integer,
    )
    cmt = Column(
        Text,
    )
    status_cn = Column(
        Integer,
    )
    app_flags = Column(
        Integer,
    )
    timestmp = Column(
        DateTime,
    )
    ncd = Column(
        String(11),
    )
    add_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    add_date = Column(DateTime)
    chg_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    chg_date = Column(
        DateTime,
    )



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        String,
                        DateTime,
                        ForeignKey,
                        )


class Vendor(Carepoint.BASE):
    __dbname__ = 'cph'
    __tablename__ = 'VEND'

    ID = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    COMPANY = Column(String)
    ADDR1 = Column(String)
    ADDR2 = Column(String)
    CITY = Column(String)
    STATE = Column(String)
    ZIP = Column(String)
    FEDID = Column(String)
    CONTACT = Column(String)
    PHONE = Column(String)
    ISSUE1099 = Column(String)
    ONETIME = Column(String)
    FAX = Column(String)
    ACCT = Column(String)
    TERMS = Column(String)
    COSTTYPE = Column(Integer)
    compound_vendor_yn = Column(String)
    POPrefix = Column(String)
    po_alpha_align_cn = Column(Integer)
    po_alpha_used = Column(String)
    po_next_id_int = Column(Integer)
    DocumentNumber = Column(String)
    NumberOfDigits = Column(Integer)
    LastPODate = Column(DateTime)
    DocCounter = Column(Integer)
    grid_color = Column(Integer)
    bold_yn = Column(Integer)
    italic_yn = Column(Integer)
    add_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    add_date = Column(DateTime)
    chg_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    chg_date = Column(DateTime)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        String,
                        DateTime,
                        Boolean,
                        ForeignKey,
                        )


class Phone(Carepoint.BASE):
    __tablename__ = 'csphone'
    __dbname__ = 'cph'

    phone_id = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    parent_phone_id = Column(
        Integer,
        ForeignKey('csphone.phone_id'),
    )
    inherited_yn = Column(Boolean)
    owner_type_cn = Column(Integer)
    area_code = Column(String)
    phone_no = Column(String)
    extension = Column(String)
    when_in_use = Column(String)
    anote = Column(String)
    app_flags = Column(Integer)
    timestmp = Column(DateTime)
    add_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    add_date = Column(DateTime)
    chg_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    chg_date = Column(DateTime)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import unittest
from sqlalchemy.schema import Table
from carepoint.tests.db.db import DatabaseTest
from carepoint.models.cph.organization_address import OrganizationAddress


class TestModelsCphOrganizationAddress(DatabaseTest):

    def test_table_initialization(self):
        self.assertIsInstance(OrganizationAddress.__table__, Table)

    def test_addr_mixin_col(self):
        self.assertTrue(
            hasattr(OrganizationAddress, 'addr_id')
        )

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import os
import unittest
import mock
from contextlib import contextmanager

from carepoint import Carepoint


class EndTestException(Exception):
    pass


class CarepointTest(unittest.TestCase):

    MODEL_DIR = os.path.join(os.path.dirname(__file__), 'test_models')

    @mock.patch('carepoint.db.carepoint.Db')
    def setUp(self, db_mock):
        self.cp_args = {
            'user': 'User',
            'passwd': 'Passwd',
            'server': 'Server',
        }
        cp = Carepoint(**self.cp_args)
        cp.register_model_dir(self.MODEL_DIR)
        self.carepoint = cp
        self.db_mock = db_mock

    def __get_model_obj(self):
        self.carepoint.find_models()
        model_obj = self.carepoint['TestModel']
        return model_obj

    @contextmanager
    def __get_mock_session(self, unwrapped=True):
        with mock.patch.object(self.carepoint, '_get_model_session') as mk:
            enter, session = mock.MagicMock(), mock.MagicMock()
            enter.return_value = enter
            mk.return_value = session
            session.__enter__.return_value = enter
            yield enter if unwrapped else mk

    #
    # Test Initializations and other core stuff
    #

    @mock.patch('carepoint.db.carepoint.Db')
    @mock.patch('carepoint.db.carepoint.super')
    def test_carepoint_init(self, sup_mk, db_mk):
        cp = Carepoint(**self.cp_args)
        sup_mk.assert_called_once_with(Carepoint, cp)

    def test_carepoint_initial_iter_refresh(self):
        self.assertFalse(
            self.carepoint.iter_refresh
        )

    def test_carepoint_assigns_instance_env(self):
        self.assertTrue(
            self.carepoint.env['cph'],
        )

    def test_cph_db_init(self):
        self.assertTrue(
            self.carepoint.dbs['cph']
        )

    def test_cph_db_assign(self):
        self.carepoint.dbs['cph'] = self.db_mock

    def test_carepoint_assigns_db_params(self):
        """ It should assign the database params as an instance var """
        self.assertIsInstance(self.carepoint.db_params, dict)

    def test_non_dir(self):
        '''
        Test to make sure that an EnvironmentError is raised with an
        invalid model dir
        '''
        with self.assertRaises(EnvironmentError):
            self.carepoint.register_model_dir(
                os.path.join(self.MODEL_DIR, 'not_a_dir'))

    def test_model_import_getitem(self):
        ''' Test if model is correctly initialized '''
        self.carepoint.find_models()
        result = self.carepoint.get('TestModel', None)
        self.assertNotEqual(result, None)

    def test_model_import_getattr(self):
        ''' Test if model is correctly initialized '''
        self.carepoint.find_models()
        self.assertIn('TestModel', self.carepoint)

    def test_model_methods(self):
        ''' Test if model is correctly initialized '''
        self.carepoint.set_iter_refresh()
        model_obj = self.carepoint['TestModel']
        self.assertTrue(model_obj.run())  # < classmethods are exposed

    def test_init_env_clear(self):
        """ It should clear the global environment when True """
        with mock.patch.object(self.carepoint, 'dbs') as dbs:
            dbs.clear.side_effect = EndTestException
            with self.assertRaises(EndTestException):
                self.carepoint._init_env(True)

    #
    # Test the session handler
    #

    def test_get_model_session_gets_session(self):
        """ It should get session for the database """
        model_obj = self.__get_model_obj()
        with mock.patch.object(self.carepoint, 'env') as env:
            with self.carepoint._get_model_session(model_obj):
                pass
            env[model_obj.__dbname__].assert_called_once_with()

    def test_get_model_session_yields_session(self):
        """ It should yield session for the database """
        model_obj = self.__get_model_obj()
        with mock.patch.object(self.carepoint, 'env') as env:
            with self.carepoint._get_model_session(model_obj) as res:
                self.assertEqual(
                    env[model_obj.__dbname__](),
                    res,
                )

    def test_get_model_session_commit(self):
        """ It should commit session after yield """
        model_obj = self.__get_model_obj()
        with mock.patch.object(self.carepoint, 'env') as env:
            with self.carepoint._get_model_session(model_obj):
                pass
            env[model_obj.__dbname__]().commit.assert_called_once_with()

    def test_get_model_session_rollback(self):
        """ It should roll session back on error """
        model_obj = self.__get_model_obj()
        with mock.patch.object(self.carepoint, 'env') as env:
            env[model_obj.__dbname__]().commit.side_effect = EndTestException
            with self.assertRaises(EndTestException):
                with self.carepoint._get_model_session(model_obj):
                    pass
            env[model_obj.__dbname__]().rollback.assert_called_once_with()

    def test_get_model_session_close(self):
        """ It should always close session """
        model_obj = self.__get_model_obj()
        with mock.patch.object(self.carepoint, 'env') as env:
            with self.carepoint._get_model_session(model_obj):
                pass
            env[model_obj.__dbname__]().close.assert_called_once_with()

    #
    # Test the SMB handlers
    #

    def test_smb_prefix(self):
        expect = 'smb://%s:%s@' % (self.cp_args['user'],
                                   self.cp_args['passwd'])
        self.assertEqual(
            expect, self.carepoint._smb_prefix,
            'SMB prefix not correct. Expect %s - Got %s' % (
                expect, self.carepoint._smb_prefix,
            )
        )

    @mock.patch('carepoint.db.carepoint.urllib2')
    @mock.patch('carepoint.db.carepoint.SMBHandler')
    def test_get_file_builds_opener(self, smb_mk, url_mk):
        self.carepoint.get_file('expect')
        url_mk.build_opener.assert_called_once_with(smb_mk)

    @mock.patch('carepoint.db.carepoint.urllib2')
    @mock.patch('carepoint.db.carepoint.SMBHandler')
    def test_get_file_opens_uri(self, smb_mk, url_mk):
        expect = 'Test'
        self.carepoint.get_file(expect)
        url_mk.build_opener().open.assert_called_once_with(
            '%s%s' % (self.carepoint._smb_prefix, expect)
        )

    @mock.patch('carepoint.db.carepoint.urllib2')
    @mock.patch('carepoint.db.carepoint.SMBHandler')
    def test_get_file_returns_opened_handler(self, smb_mk, url_mk):
        expect = 'Test'
        res = self.carepoint.get_file(expect)
        self.assertEqual(
            url_mk.build_opener().open(), res,
        )

    @mock.patch('carepoint.db.carepoint.urllib2')
    @mock.patch('carepoint.db.carepoint.SMBHandler')
    def test_send_file_builds_opener(self, smb_mk, url_mk):
        self.carepoint.send_file('expect', '')
        url_mk.build_opener.assert_called_once_with(smb_mk)

    @mock.patch('carepoint.db.carepoint.urllib2')
    @mock.patch('carepoint.db.carepoint.SMBHandler')
    def test_send_file_sends_file(self, smb_mk, url_mk):
        expect = 'Test'
        data = 'data'
        self.carepoint.send_file(expect, data)
        url_mk.build_opener().__enter__().open.assert_called_once_with(
            '%s%s' % (self.carepoint._smb_prefix, expect), data=data,
        )

    @mock.patch('carepoint.db.carepoint.urllib2')
    @mock.patch('carepoint.db.carepoint.SMBHandler')
    def test_send_file_returns_true(self, smb_mk, url_mk):
        res = self.carepoint.send_file('expect', '')
        self.assertTrue(res)

    #
    # Test the database convenience handlers
    #

    # Read

    def test_read_calls_query_with_model_obj(self):
        model_obj = self.__get_model_obj()
        record_id = 1
        with self.__get_mock_session() as mk:
            self.carepoint.read(model_obj, record_id)
            mk.query.assert_called_once_with(model_obj)

    def test_read_calls_get_with_record_id(self):
        model_obj = self.__get_model_obj()
        record_id = 1
        with self.__get_mock_session() as mk:
            mk.query.return_value = query_mk = mock.MagicMock()
            self.carepoint.read(model_obj, record_id)
            query_mk.get.assert_called_once_with(record_id)

    def test_read_returns_response(self):
        model_obj = self.__get_model_obj()
        record_id = 1
        expect = 'Response'
        with self.__get_mock_session() as mk:
            mk.query.return_value = query_mk = mock.MagicMock()
            query_mk.get.return_value = expect
            response = self.carepoint.read(model_obj, record_id)
            self.assertEqual(response, expect)

    # Search
    def test_search_calls_query_with_model_obj(self):
        filters = {'test_col': 'Test'}
        with self.__get_mock_session() as mk:
            model_obj = self.__get_model_obj()
            self.carepoint.search(model_obj, filters)
            mk.query.assert_called_once_with(model_obj)

    def test_search_calls_filter_with_filters(self):
        model_obj = self.__get_model_obj()
        filters = {'test_col': 'Test'}
        with self.__get_mock_session() as mk:
            mk.query.return_value = query_mk = mock.MagicMock()
            self.carepoint.search(model_obj, filters)
            query_mk.filter.assert_called_once_with(
                *self.carepoint._unwrap_filters(model_obj, filters)
            )

    def test_search_calls_filter_when_no_filters(self):
        model_obj = self.__get_model_obj()
        with self.__get_mock_session() as mk:
            mk.query.return_value = query_mk = mock.MagicMock()
            self.carepoint.search(model_obj)
            query_mk.filter.assert_called_once_with(**{})

    def test_search_returns_response(self):
        model_obj = self.__get_model_obj()
        filters = {'test_col': 'Test'}
        with self.__get_mock_session() as mk:
            with mock.patch.object(self.carepoint, 'read') as read_mk:
                response = self.carepoint.read(model_obj, filters)
                self.assertEqual(
                    read_mk(),
                    response,
                )

    # Get Next Sequence

    def test_get_next_sequence_session(self):
        """ It should get session for db """
        expect = 'expect'
        with mock.patch.object(self.carepoint, '_get_session') as mk:
            res = self.carepoint.get_next_sequence(None, expect)
            mk.assert_called_once_with(expect)

    @mock.patch('carepoint.db.carepoint.text')
    def test_get_next_sequence_execute(self, text):
        """ It should execute stored procedure on connection """
        expect = 'expect'
        with mock.patch.object(self.carepoint, '_get_session') as mk:
            self.carepoint.get_next_sequence(expect)
            mk().__enter__().connection().execute.assert_called_once_with(
                text(), seq_name=expect,
            )

    def test_get_next_sequence_fetch(self):
        """ It should return result of fetch """
        with mock.patch.object(self.carepoint, '_get_session') as mk:
            res = self.carepoint.get_next_sequence(None)
            expect = mk().__enter__().connection().execute().fetchall()[0][0]
            self.assertEqual(
                expect, res,
            )

    # Create
    def test_create_calls_get_model_session_with_model_obj(self):
        model_obj = mock.MagicMock()
        with self.__get_mock_session(False) as mk:
            self.carepoint.create(model_obj, {})
            mk.assert_called_once_with(model_obj)

    def test_create_initializes_new_model_obj_with_vals(self):
        model_obj = mock.MagicMock()
        vals = {'test_col': 'Test'}
        with self.__get_mock_session() as mk:
            self.carepoint.create(model_obj, vals)
            model_obj.assert_called_once_with(**vals)

    def test_create_adds_new_record_to_session(self):
        model_obj = mock.MagicMock()
        response_expect = 'ResponseExpect'
        model_obj.return_value = response_expect
        vals = {'test_col': 'Test'}
        with self.__get_mock_session() as mk:
            self.carepoint.create(model_obj, vals)
            mk.add.assert_called_once_with(response_expect)

    def test_create_returns_new_record(self):
        model_obj = mock.MagicMock()
        response_expect = 'ResponseExpect'
        model_obj.return_value = response_expect
        vals = {'test_col': 'Test'}
        with self.__get_mock_session() as mk:
            response = self.carepoint.create(model_obj, vals)
            self.assertEqual(response, response_expect)

    # Update
    def test_update_calls_get_model_session_with_model_obj(self):
        model_obj = self.__get_model_obj()
        record_id = 1
        with self.__get_mock_session(False) as mk:
            with mock.patch.object(self.carepoint, 'read'):
                self.carepoint.update(model_obj, record_id, {})
                mk.assert_called_once_with(model_obj)

    def test_update_calls_read_with_model_and_record_id(self):
        model_obj = self.__get_model_obj()
        record_id = 1
        vals = {'test_col': 'Test'}
        with self.__get_mock_session():
            with mock.patch.object(self.carepoint, 'read') as read_mk:
                self.carepoint.update(model_obj, record_id, vals)
                read_mk.assert_called_once_with(model_obj, record_id)

    def test_update_calls_update_with_vals(self):
        model_obj = self.__get_model_obj()
        record_id = 1
        vals = {'test_col': 'Test'}
        with self.__get_mock_session() as mk:
            mk.query.return_value = query_mk = mock.MagicMock()
            query_mk.get.return_value = get_mk = mock.MagicMock()
            self.carepoint.update(model_obj, record_id, vals)
            self.assertEqual(
                mk().query().get().test_col,
                get_mk.test_col,
                'Record attribute was not updated. Expect Test, Got %s' % (
                    get_mk.test_col,
                )
            )

    def test_update_returns_record(self):
        model_obj = self.__get_model_obj()
        record_id = 1
        vals = {'test_col': 'Test'}
        with self.__get_mock_session() as mk:
            with mock.patch.object(self.carepoint, 'read') as read:
                response = self.carepoint.update(model_obj, record_id, vals)
                self.assertEqual(read(), response)

    # Delete
    def test_delete_calls_get_model_session_with_model_obj(self):
        model_obj = self.__get_model_obj()
        record_id = 1
        with self.__get_mock_session(False) as mk:
            with mock.patch.object(self.carepoint, 'read') as read_mk:
                read_mk.return_value = read_mk
                read_mk.count.return_value = 0
                self.carepoint.delete(model_obj, record_id)
                mk.assert_called_once_with(model_obj)

    def test_delete_calls_read_with_model_and_record_id(self):
        model_obj = self.__get_model_obj()
        record_id = 1
        with self.__get_mock_session():
            with mock.patch.object(self.carepoint, 'read') as read_mk:
                read_mk.return_value = read_mk
                read_mk.count.return_value = 0
                self.carepoint.delete(model_obj, record_id)
                read_mk.assert_called_once_with(model_obj, record_id)

    def test_delete_asserts_result_count_eq_1(self):
        model_obj = self.__get_model_obj()
        record_id = 1
        with self.__get_mock_session():
            with mock.patch.object(self.carepoint, 'read') as read_mk:
                read_mk.return_value = read_mk
                read_mk.count.return_value = 2
                with self.assertRaises(AssertionError):
                    self.carepoint.delete(model_obj, record_id)

    def test_delete_calls_session_delete_with_record_result(self):
        model_obj = self.__get_model_obj()
        record_id = 1
        expect_return = mock.MagicMock()
        with self.__get_mock_session() as mk:
            with mock.patch.object(self.carepoint, 'read') as read_mk:
                read_mk.return_value = expect_return
                expect_return.count.return_value = 1
                self.carepoint.delete(model_obj, record_id)
                mk.delete.assert_called_once_with(expect_return)

    def test_delete_returns_false_on_no_records(self):
        model_obj = self.__get_model_obj()
        record_id = 1
        with self.__get_mock_session():
            with mock.patch.object(self.carepoint, 'read') as read_mk:
                read_mk.return_value = read_mk
                read_mk.count.return_value = 0
                response = self.carepoint.delete(model_obj, record_id)
                self.assertFalse(response)

    def test_delete_returns_true_on_delete(self):
        model_obj = self.__get_model_obj()
        record_id = 1
        with self.__get_mock_session():
            with mock.patch.object(self.carepoint, 'read') as read_mk:
                read_mk.return_value = read_mk
                read_mk.count.return_value = 1
                response = self.carepoint.delete(model_obj, record_id)
                self.assertTrue(response)

    #
    # Test filter criterion generators
    #

    def __setup_criterion_test(self, operator, query='Query'):
        self.carepoint.find_models()
        model_obj = self.carepoint['TestModel']
        attr_name = 'test_col'
        col_obj = getattr(model_obj, attr_name)
        return [col_obj, model_obj, attr_name, operator, query]

    def test_create_criterion_returns_correct_double_eq(self):
        args = self.__setup_criterion_test('==')
        self.assertEqual(
            self.carepoint._create_criterion(*args[1:]),
            args[0] == args[-1]
        )

    def test_create_criterion_returns_correct_eq(self):
        args = self.__setup_criterion_test('=')
        self.assertEqual(
            self.carepoint._create_criterion(*args[1:]),
            args[0] == args[-1]
        )

    def test_create_criterion_returns_correct_lt(self):
        args = self.__setup_criterion_test('<')
        self.assertEqual(
            self.carepoint._create_criterion(*args[1:]),
            args[0] < args[-1]
        )

    def test_create_criterion_returns_correct_le(self):
        args = self.__setup_criterion_test('<=')
        self.assertEqual(
            self.carepoint._create_criterion(*args[1:]),
            args[0] <= args[-1]
        )

    def test_create_criterion_returns_correct_gt(self):
        args = self.__setup_criterion_test('>')
        self.assertEqual(
            self.carepoint._create_criterion(*args[1:]),
            args[0] > args[-1]
        )

    def test_create_criterion_returns_correct_ge(self):
        args = self.__setup_criterion_test('>=')
        self.assertEqual(
            self.carepoint._create_criterion(*args[1:]),
            args[0] >= args[-1]
        )

    def test_create_criterion_raises_not_implemented_error(self):
        args = self.__setup_criterion_test('===')
        with self.assertRaises(KeyError):
            self.carepoint._create_criterion(*args[1:])

    def test_create_criterion_raises_attribute_error(self):
        args = self.__setup_criterion_test('==')
        args[2] = 'Nope'
        with self.assertRaises(AttributeError):
            self.carepoint._create_criterion(*args[1:])

    #
    # Test filter dictionary unwrapping
    #

    def test_unwrap_filters_empty(self):
        model_obj = self.__get_model_obj()
        expect = []
        self.assertEqual(
            self.carepoint._unwrap_filters(model_obj, {}), expect,
        )

    def test_unwrap_filters_none(self):
        model_obj = self.__get_model_obj()
        expect = []
        self.assertEqual(
            self.carepoint._unwrap_filters(model_obj), expect,
        )

    def test_unwrap_filters_dict(self):
        model_obj = self.__get_model_obj()
        col_expect = 'test_col'
        query_expect = ['test_1', 'test_2', ]
        ops_expect = ['>=', '==', ]
        filters = {
            col_expect: {
                ops_expect[0]: query_expect[0],
                ops_expect[1]: query_expect[1],
            }
        }
        expect = [
            mock.call(model_obj, col_expect, ops_expect[0], query_expect[0]),
            mock.call(model_obj, col_expect, ops_expect[1], query_expect[1]),
        ]
        with mock.patch.object(self.carepoint, '_create_criterion') as mk:
            self.carepoint._unwrap_filters(model_obj, filters)
            mk.assert_has_calls(expect, any_order=True)

    @mock.patch('carepoint.db.carepoint.or_')
    def test_unwrap_filters_list(self, or_):
        """ It should create a proper or query for lists. """
        model_obj = self.__get_model_obj()
        col_expect = 'test_col'
        query_expect = ['test_1', 'test_2']
        filters = {
            col_expect: query_expect
        }
        expect = [
            mock.call(model_obj, col_expect, '==', query_expect[0]),
            mock.call(model_obj, col_expect, '==', query_expect[1]),
        ]
        criterion_return = 'Criterion'
        with mock.patch.object(self.carepoint, '_create_criterion') as mk:
            mk.return_value = criterion_return
            self.carepoint._unwrap_filters(model_obj, filters)
            mk.assert_has_calls(expect, any_order=True)
            or_.assert_called_once_with(criterion_return, criterion_return)

    def test_unwrap_filters_str(self):
        model_obj = self.__get_model_obj()
        col_expect = 'test_col'
        query_expect = 'test_1'
        op_expect = '=='
        filters = {col_expect: query_expect}
        with mock.patch.object(self.carepoint, '_create_criterion') as mk:
            self.carepoint._unwrap_filters(model_obj, filters)
            mk.assert_called_once_with(
                model_obj, col_expect, op_expect, query_expect
            )

    def test_unwrap_filters_returns_list(self):
        model_obj = self.__get_model_obj()
        self.assertIsInstance(
            self.carepoint._unwrap_filters(model_obj), list
        )

    #
    # Test dictionary overrides for model lookups
    #

    def test_iter_init_empty(self):
        self.assertEqual(len([i for i in self.carepoint]), 0)

    def test_values_init_empty(self):
        self.assertEqual(len(self.carepoint.values()), 0)

    def test_keys_init_empty(self):
        self.assertEqual(len(self.carepoint.keys()), 0)

    def test_items_init_empty(self):
        self.assertEqual(len(self.carepoint.items()), 0)

    def test_iteritems_init_empty(self):
        self.assertEqual(len([i for i in self.carepoint.iteritems()]), 0)

    def test_itervalues_init_empty(self):
        self.assertEqual(len([i for i in self.carepoint.itervalues()]), 0)

    def test_iterkeys_init_empty(self):
        self.assertEqual(len([i for i in self.carepoint.iterkeys()]), 0)

    def test_set_iter_refresh(self):
        self.carepoint.set_iter_refresh()
        self.assertTrue(self.carepoint.iter_refresh)

    def test_iter_after_refresh(self):
        self.carepoint.set_iter_refresh()
        self.assertEqual(len([i for i in self.carepoint]), 1)

    def test_values_after_refresh(self):
        self.carepoint.set_iter_refresh()
        self.assertEqual(len(self.carepoint.values()), 1)

    def test_keys_after_refresh(self):
        self.carepoint.set_iter_refresh()
        self.assertEqual(len(self.carepoint.keys()), 1)

    def test_items_after_refresh(self):
        self.carepoint.set_iter_refresh()
        self.assertEqual(len(self.carepoint.items()), 1)

    def test_iteritems_after_refresh(self):
        self.carepoint.set_iter_refresh()
        self.assertEqual(len([i for i in self.carepoint.iteritems()]), 1)

    def test_itervalues_after_refresh(self):
        self.carepoint.set_iter_refresh()
        self.assertEqual(len([i for i in self.carepoint.itervalues()]), 1)

    def test_iterkeys_after_refresh(self):
        self.carepoint.set_iter_refresh()
        self.assertEqual(len([i for i in self.carepoint.iterkeys()]), 1)

    def test_wrong_model_getitem(self):
        ''' Test to verify that a KeyError is raised for invalid model name '''
        with self.assertRaises(KeyError):
            self.carepoint.set_iter_refresh()
            self.carepoint['ThisIsNotAModelThatExists']

    def test_wrong_model_getattr(self):
        ''' Test to verify that a KeyError is raised for invalid model name '''
        with self.assertRaises(AttributeError):
            self.carepoint.set_iter_refresh()
            self.carepoint.ThisIsNotAModelThatExists

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright 2016 LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import enum

from sqlalchemy import (Column,
                        Integer,
                        DateTime,
                        ForeignKey,
                        )
from sqlalchemy.types import Enum
from sqlalchemy.ext.declarative import declared_attr


class EnumPhoneType(enum.Enum):
    """ It provides PEP-0435 compliant Carepoint Phone Type Enumerable """

    assistant = 1
    business = 2
    business_2 = 3
    business_fax = 4
    car = 5
    home = 6
    home_2 = 7
    home_fax = 8
    mobile = 9
    beeper = 10
    other_fax = 11


class PhoneMixin(object):
    """ This is a mixin for Phone Many2Many bindings """

    priority = Column(Integer)
    phone_type_cn = Column(Integer)
    app_flags = Column(Integer)
    timestmp = Column(DateTime)

    @declared_attr
    def phone_id(cls):
        return Column(
            Integer,
            ForeignKey('csphone.phone_id'),
            primary_key=True,
        )

    @property
    def phone_type(self):
        return EnumPhoneType(self.phone_type_cn)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        String,
                        DateTime,
                        ForeignKey,
                        Text,
                        )


class PatientDisease(Carepoint.BASE):
    __tablename__ = 'cppat_dx'
    __dbname__ = 'cph'

    ptdx_id = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    pat_id = Column(
        Integer,
        ForeignKey('cppat.pat_id'),
    )
    fdbdx = Column(String)
    icd9 = Column(String)
    name = Column(String)
    dx_date = Column(DateTime)
    caring_md_id = Column(
        Integer,
        ForeignKey('cpmd.md_id'),
    )
    onset_date = Column(DateTime)
    resolution_status_cn = Column(Integer)
    screen_yn = Column(Integer)
    mar_cmt = Column(Text)
    cmt = Column(Text)
    status_cn = Column(Integer)
    app_flags = Column(Integer)
    timestmp = Column(DateTime)
    primary_yn = Column(Integer)
    dxid = Column(Integer)
    icd10 = Column(String)
    icd10_name = Column(String)
    add_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    add_date = Column(DateTime)
    chg_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    chg_date = Column(DateTime)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import unittest
import mock
from urllib import quote_plus as urlquote
from carepoint.db import Db


class TestDb(unittest.TestCase):

    def setUp(self, ):
        self.params = {
            'usr': 'This is a user',
            'pass': '23243dsc dxcsdkc ewr239Copyright',
            'srv': 'server',
            'drv': 'drv',
            'db': 'db',
            'prt': 'port',
        }
        self.dsn = 'mssql+pyodbc://{usr}:{pass}@{srv}:{prt}/{db}?driver={drv}'

    @mock.patch('carepoint.db.db.create_engine')
    def test_mssql_dsn(self, mk):
        Db(
            self.params['srv'],
            self.params['usr'],
            self.params['pass'],
            self.params['db'],
            self.params['prt'],
            self.params['drv'],
        )
        self.params['usr'] = urlquote(self.params['usr'])
        self.params['pass'] = urlquote(self.params['pass'])
        mk.assert_called_once_with(self.dsn.format(**self.params))

    @mock.patch('carepoint.db.db.create_engine')
    def test_mssql_dsn_return(self, mk):
        res = Db(
            self.params['srv'],
            self.params['usr'],
            self.params['pass'],
            self.params['db'],
            self.params['prt'],
            self.params['drv'],
        )
        self.assertEqual(mk(), res)

    @mock.patch('carepoint.db.db.create_engine')
    def test_sqlite_dsn(self, mk):
        Db(drv='sqlite')
        mk.assert_called_once_with('sqlite://')

    @mock.patch('carepoint.db.db.create_engine')
    def test_sqlite_dsn_return(self, mk):
        res = Db(drv='sqlite')
        self.assertEqual(mk(), res)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from mixer.backend.sqlalchemy import mixer
from datetime import datetime


dt_now = datetime.now()
__model__ = 'carepoint.models.cph.patient.Patient'


patient_default = mixer.blend(
    __model__,
    pat_id=1,
    cmt_id=1,
    pat_status_cn=1,
    pat_type_cn=1,
    nh_pat_id='NhPatId',
    chart_id='ChartId',
    lname='LName',
    lname_sdx='LNameSdx',
    mname='Mname',
    title_lu='TitleLu',
    suffix_lu='SuffixLu',
    alias='Alias',
    mmname='MmName',
    alt1_id='AltId',
    pref_meth_cont_cn='PrefMeth',
    best_cont_time='BestCon',
    ssn='Social',
    dln='Dln',
    dln_state_cd='DlnStateCd',
    email='Email',
    gender_cd='Gender',
    birth_date=dt_now,
    death_date=dt_now,
    no_safety_caps_yn=False,
    generics_yn=False,
    label_style_cn=1,
    primary_md_id=1,
    secondary_md_id=1,
    edu_level_cn=1,
    ethnicity_cd='Ethnicity',
    marital_status_cd='MaritalStatus',
    religion_cn=1,
    name_spouse='NameSpous',
    primary_lang_cd='PrimaryLang',
    rec_release_status_cn=1,
    rec_release_date=dt_now,
    hh_relation_cn=1,
    hh_pat_id=1,
    fam_relation_cn='FamRelation',
    fam_pat_id=1,
    primary_store_id=1,
    bad_check_yn=False,
    price_formula_id=1,
    cmt='Cmt',
    status_cn=1,
    facility_pat_yn=False,
    conv_code='ConvCode',
    app_flags=1,
    timestmp=dt_now,
    comp_cn=1,
    hp_blnAdmExt=1,
    hp_ExtAtt=1,
    user_def_1='UserDef',
    user_def_2='UserDef',
    user_def_3='UserDef',
    user_def_4='UserDef',
    sc_pat_id=1,
    resp_party_id=1,
    auto_refill_cn=1,
    fill_yn=False,
    fill_stop_date=dt_now,
    fill_resume_date=dt_now,
    fill_stop_reason_cn=1,
    fill_stop_user_id=1,
    registration_date=dt_now,
    anonymous_yn=False,
    market_yn=False,
    representative='Represent',
    discharge_date=dt_now,
    do_not_resuscitate_yn=False,
    discharge_exp_date=dt_now,
    pat_loc_cn=1,
    rx_priority_default_cn=1,
    ship_cn=1,
    residence_cn=1,
    add_user_id=1,
    add_date=dt_now,
    chg_user_id=1,
    chg_date=dt_now,
)


def patient_rnd(cnt):
    return mixer.cycle(cnt).blend(__model__)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from mixer.backend.sqlalchemy import mixer
from datetime import datetime

dt_now = datetime.now()
__model__ = 'carepoint.models.cph.dispense.Dispense'


dispense_default = mixer.blend(
    __model__,
    rx_id=1,
    store_id=1,
    disp_ndc='DispNdc',
    disp_drug_name='DispDrugName',
    prod_expire_date=dt_now,
    mfg='Mfg',
    orig_mfg='OrifMfg',
    pkg_size=1,
    rxdisp_id=1,
    fill_no=1,
    dispense_date=dt_now,
    dispense_qty=1,
    disp_days_supply=1,
    sig_text='SigText',
    sched_of_admin_cn=1,
    freq_of_admin=1,
    units_per_dose=1,
    lot_number='LotNumber',
    disp_awp=1,
    disp_aac=1,
    disp_mac=1,
    disp_ful=1,
    disp_udef=1,
    tech_initials='TechInitials',
    rph_initials='RphInitials',
    cnsl_initials='CnslInitials',
    icd9='Icd',
    daw_disp_cn=1,
    level_of_service=1,
    cmt='cmt',
    status_cn=1,
    alt_pick_up_id=1,
    alt_pick_up_cn=1,
    clarification_fill=1,
    trip_no=1,
    gpi_disp=1,
    label_3pty_yn=False,
    reject_3pty_yn=False,
    pay_type_cn=1,
    price_differs_yn=False,
    pat_loc_cn=1,
    billing_units='BillingUnits',
    price_table_id=1,
    app_flags=1,
    timestmp=dt_now,
    price_meth_cn=1,
    uu='UU',
    p_tbl_override_yn=False,
    label_id=1,
    billing_hold=1,
    post_bal_yn=False,
    inv_no='InvNo',
    disp_qty_delta=1,
    brand_med_nec_yn=False,
    Other_coverage_cd=1,
    epsdt_yn=False,
    exempt_cd=1,
    num_labels=1,
    location='Location',
    qty_override=1,
    dur_summary='DurSummary',
    cov_overrides=1,
    use_secondary_ins_yn=False,
    hp_blnRxExtr=1,
    item_id=1,
    ud_override='UdOverride',
    disp_status_cn=1,
    order_id=1,
    sig_id=1,
    counsel_yn=1,
    processing_date=dt_now,
    hcpcs_mod_cn=1,
    acct_id=1,
    track_pat_resp_yn=1,
    track_org_resp_yn=1,
    label_fac_id=1,
    extern_process_cn=1,
    price_ovr_user_id=1,
    price_ovr_reason_cn=1,
    wf_status_cn=1,
    verify_user_id=1,
    verify_timestamp=dt_now,
    ar_hold=1,
    disp_type_cn=1,
    own_use_pricing_yn=1,
    pos_processed_yn=1,
    prescriber_cn=1,
    visit_nbr='VisitNbr',
    disp_udef2=1,
    SecondaryICD9='SecondaryIcd',
    uandc_pricing_used_yn=1,
    special_pkg_ind_cn=1,
    delay_reason_cn=1,
    place_of_service_cn=1,
    pat_residence_cn=1,
    compound_type_cn=1,
    pharmacy_service_type_cn=1,
    admin_start_date=dt_now,
    ClariFill_2=1,
    ClariFill_3=1,
    other_coverage_cd_2=1,
    pat_assign_ind_yn=1,
    prov_assign_ind_yn=1,
    route_of_admin_ovr='RouteAdmin',
    csr_pickup_id=1,
    icd10='icd',
    secondaryicd10='secondaryicd',
    add_user_id=1,
    add_date=dt_now,
    chg_user_id=1,
    chg_date=dt_now,
)


def dispense_rnd(cnt):
    return mixer.cycle(cnt).blend(__model__)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import unittest
from sqlalchemy.schema import Table
from carepoint.tests.db.db import DatabaseTest
from carepoint.models.cph.dispense_price import DispensePrice


class TestModelsCphDispensePrice(DatabaseTest):

    def test_table_initialization(self, ):
        self.assertIsInstance(DispensePrice.__table__, Table)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from mixer.backend.sqlalchemy import mixer
from datetime import datetime


dt_now = datetime.now()
__model__ = 'carepoint.models.cph.prescription.Prescription'


presription_default = mixer.blend(
    __model__,
    rx_id=1,
    script_no='ScriptNo',
    old_script_no='OldScriptNo',
    pat_id=1,
    store_id=1,
    md_id=1,
    ndc='Ndc',
    gcn_seqno=1,
    mfg='Mfg',
    drug_name='DrugName',
    input_src_cn=1,
    src_pat_meds_cn=1,
    start_date=dt_now,
    expire_date=dt_now,
    stop_date=dt_now,
    sig_code='SigCode',
    sig_text='SigText',
    written_qty=1,
    starter_qty=1,
    days_supply=1,
    pkg_size=1,
    units_per_dose=1,
    units_entered='UnitsEntered',
    awp=1,
    udef=1,
    ful=1,
    mac=1,
    aac=1,
    freq_entered='FreqEntered',
    freq_of_admin=1,
    sched_of_admin_cn=1,
    daw_yn=False,
    refills_orig=1,
    refills_left=1,
    last_rxdisp_id=1,
    last_refill_qty=1,
    last_refill_date=dt_now,
    refill_date=dt_now,
    src_org_id=1,
    cmt='Cmt',
    exit_state_cn=1,
    script_status_cn=1,
    sched_date=dt_now,
    sched_time=dt_now,
    scheduled_yn=False,
    drug_dea_class='DrugDeaClass',
    manual_add_yn=False,
    status_cn=1,
    life_date=dt_now,
    self_prescribed_yn=False,
    last_transfer_type_io='LastTransferType',
    last_disp_prod='LastDsipProd',
    transfer_cnt=1,
    wc_claim_id='WcClaimId',
    injury_date=dt_now,
    gpi_rx=1,
    auth_by='AuthBy',
    orig_disp_date=dt_now,
    short_term_yn=1,
    orig_date=dt_now,
    refills_used=1,
    wc_emp_id=1,
    dose_unit='DoseUnit',
    dosage_multiplier=1,
    df='Df',
    uu='Uu',
    app_flags=1,
    timestmp=dt_now,
    sched2_no='SchedTwo',
    orig_expire_date=dt_now,
    rxq_status_cn=1,
    IVRCmt='IVRCmt',
    wc_bodypart='WcBodypart',
    comp_ndc='CompNdc',
    treatment_yn=False,
    ivr_callback='IvrCallback',
    autofill_yn=False,
    autofill_resume_date=dt_now,
    workflow_status_cn=1,
    extern_process_cn=1,
    md_fac_id=1,
    owner_store_id=1,
    study_id=1,
    min_days_until_refill=1,
    sig_text_english='SigTextEnglish',
    order_fullfillment_cn=1,
    taxable=1,
    MAR_flag=1,
    FreeFormStrength='FreeFormStrength',
    priority_cn=1,
    # ecs_yn=False,
    # edit_locked_yn=False,
    # locked_yn=False,
    # locked_id=1,
    # locked_user_id=1,
    daw_rx_cn=1,
    add_user_id=1,
    add_date=dt_now,
    chg_user_id=1,
    chg_date=dt_now,
)


def prescription_rnd(cnt):
    return mixer.cycle(cnt).blend(__model__)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        String,
                        Boolean,
                        )


class FdbNdc(Carepoint.BASE):
    __tablename__ = 'fdrndc'
    __dbname__ = 'cph'

    ndc = Column(String, primary_key=True)
    id = Column(Integer)
    lblrid = Column(String)
    gcn_seqno = Column(Integer)
    ps = Column(Integer)
    df = Column(Integer)
    ad = Column(String)
    ln = Column(String)
    bn = Column(String)
    pndc = Column(Integer)
    repndc = Column(Integer)
    ndcfi = Column(Integer)
    daddnc = Column(String)
    dupdc = Column(String)
    desi = Column(String)
    desdtec = Column(Integer)
    desi2 = Column(String)
    des2dtec = Column(Integer)
    dea = Column(Integer)
    cl = Column(String)
    gpi = Column(Integer)
    hosp = Column(Integer)
    innov = Column(Integer)
    ipi = Column(Integer)
    mini = Column(Integer)
    maint = Column(Integer)
    obc = Column(String)
    obsdtec = Column(String)
    ppi = Column(Integer)
    stpk = Column(Integer)
    repack = Column(Integer)
    top200 = Column(Integer)
    ud = Column(Integer)
    csp = Column(Integer)
    color = Column(String)
    flavor = Column(String)
    shape = Column(String)
    ndl_gdge = Column(Integer)
    ndl_lngth = Column(Integer)
    syr_cpcty = Column(Integer)
    shlf_pck = Column(Integer)
    shipper = Column(Integer)
    skey = Column(String)
    hcfa_fda = Column(String)
    hcfa_unit = Column(String)
    hcfa_ps = Column(Integer)
    hcfa_appc = Column(Integer)
    hcfa_mrkc = Column(Integer)
    hcfa_trmc = Column(Integer)
    hcfa_typ = Column(Integer)
    hcfa_desc1 = Column(Integer)
    hcfa_desi1 = Column(Integer)
    uu = Column(Integer)
    pd = Column(String)
    ln25 = Column(String)
    ln25i = Column(Integer)
    gpidc = Column(Integer)
    bbdc = Column(Integer)
    home = Column(Integer)
    inpcki = Column(Integer)
    outpcki = Column(Integer)
    obc_exp = Column(String)
    ps_equiv = Column(Integer)
    plblr = Column(Integer)
    hcpc = Column(String)
    top50gen = Column(Integer)
    obc3 = Column(String)
    gmi = Column(Integer)
    gni = Column(Integer)
    gsi = Column(Integer)
    gti = Column(Integer)
    ndcgi1 = Column(Integer)
    user_gcdf = Column(String)
    user_str = Column(String)
    real_product_yn = Column(Boolean)
    no_update_yn = Column(Boolean)
    no_prc_update_yn = Column(Boolean)
    user_product_yn = Column(Integer)
    cpname_short = Column(String)
    status_cn = Column(String)
    update_yn = Column(Boolean)
    active_yn = Column(Boolean)
    ln60 = Column(String)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import enum

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        String,
                        DateTime,
                        ForeignKey,
                        Text,
                        )


class EnumUserType(enum.Enum):
    """ It provides PEP-0435 compliant Carepoint User Type Enumerable """

    user = 'U'
    group = 'G'
    machine = 'M'


class User(Carepoint.BASE):
    __tablename__ = 'csuser'
    __dbname__ = 'cph'

    user_id = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    user_type_cd = Column(String)
    login_name = Column(String)
    lname = Column(String)
    fname = Column(String)
    mname = Column(String)
    title_lu = Column(String)
    suffix_lu = Column(String)
    degree_lu = Column(String)
    initials = Column(String)
    job_title_lu = Column(String)
    anote = Column(String)
    employee_no = Column(String)
    ssn = Column(String)
    last_login_date = Column(DateTime)
    email = Column(String)
    db_password = Column(String)
    es_password = Column(String)
    cmt = Column(Text)
    system_yn = Column(Integer)
    license_no = Column(String)
    license_state_cd = Column(String)
    status_cn = Column(Integer)
    app_flags = Column(Integer)
    timestmp = Column(DateTime)
    must_change_password_yn = Column(Integer)
    password_date = Column(DateTime)
    add_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    add_date = Column(DateTime)
    chg_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    chg_date = Column(DateTime)

    @property
    def user_type(self):
        return EnumUserType(self.user_type_cd)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        String,
                        )


class OrderShip(Carepoint.BASE):
    __tablename__ = 'csom_ship'
    __dbname__ = 'cph'

    order_id = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    processing_flag = Column(Integer)
    tracking_code = Column(String)
    Bill_Name = Column(String)
    bill_addr1 = Column(String)
    bill_addr2 = Column(String)
    bill_addr3 = Column(String)
    bill_city = Column(String)
    bill_state_cd = Column(String)
    bill_zip = Column(String)
    bill_country_cd = Column(String)
    ship_name = Column(String)
    ship_addr1 = Column(String)
    ship_addr2 = Column(String)
    ship_addr3 = Column(String)
    ship_city = Column(String)
    ship_state_cd = Column(String)
    ship_zip = Column(String)
    ship_country_cd = Column(String)
    ship_phone = Column(String)
    ship_email = Column(String)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        Boolean,
                        DateTime,
                        ForeignKey,
                        )


class OrderLine(Carepoint.BASE):
    __tablename__ = 'CsOmLine'
    __dbname__ = 'cph'

    line_id = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    order_id = Column(
        Integer,
        ForeignKey('CsOm.order_id'),
    )
    rx_id = Column(
        Integer,
        ForeignKey('cprx.rx_id'),
    )
    rxdisp_id = Column(
        Integer,
        ForeignKey('cprx_disp.rxdisp_id'),
    )
    line_state_cn = Column(Integer)
    line_status_cn = Column(Integer)
    hold_yn = Column(Boolean)
    add_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    add_date = Column(DateTime)
    chg_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    chg_date = Column(DateTime)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        ForeignKey,
                        )


class Account(Carepoint.BASE):
    __tablename__ = 'cp_acct'
    __dbname__ = 'cph'

    pat_id = Column(
        Integer,
        ForeignKey('cppat.pat_id'),
        primary_key=True,
    )
    ID = Column(
        Integer,
        primary_key=True,
    )
    acct_type_cn = Column(Integer)
    resp_pty_yn = Column(Integer)
    chromis_id = Column(Integer)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from mixer.backend.sqlalchemy import mixer
from datetime import datetime


dt_now = datetime.now()
__model__ = 'carepoint.models.cph.item.Item'


item_default = mixer.blend(
    __model__,
    ITEMMSTR='ItmeStr',
    DESCR='Descr',
    TAXABLE='Taxable',
    CLASS='Class',
    UOFMSALES='UofmSales',
    UOFMORDERS='UofmOrders',
    FACTOR=1.1,
    ONHAND=1,
    ONORDER=1,
    VENDOR='Vendor',
    COST=1,
    INQTY=1,
    INDATE=dt_now,
    OUTQTY=1,
    OUTDATE=dt_now,
    ADJQTY=1,
    ADJDATE=dt_now,
    MTD_ISSUE=1,
    MTD_RCPTS=1,
    MTD_ADJ=1,
    COMMENTS='Comments',
    IMIN=1,
    IMAX=1,
    UPCCODE='UpcCode',
    POSLOC='PosLoc',
    VENDITEMNO='VendItemMo',
    EXCLUDE='Exclude',
    NDC='Ndc',
    SKU='Sku',
    AUTO_ORD=1,
    FIXED_QTY=1,
    ACTIVE_YN=1,
    AVG_UNIT_COST=1,
    location='location',
    item_id=1,
    store_id=1,
    chemical_id=1,
    allocated=1,
    machine_id=1,
    special_pkg_ind_cn=1,
    refrig_cn=1,
    order_multiples_of=1,
    add_user_id=1,
    add_date=dt_now,
    chg_user_id=1,
    chg_date=dt_now,
)


def item_rnd(cnt):
    return mixer.cycle(cnt).blend(__model__)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy.orm import relationship
from sqlalchemy import (Column,
                        String,
                        Numeric,
                        )


class FdbImg(Carepoint.BASE):
    __tablename__ = 'fdbrimgimg'
    __dbname__ = 'cph'

    IMGID = Column(
        Numeric(10, 0),
        primary_key=True,
    )
    IMGFILENM = Column(
        String,
    )
    IMAGE_ROOTS = relationship(
        'StoreParam',
        primaryjoin=""" and_(
            remote(literal_column('1')) == foreign(StoreParam.store_id),
            'SC_MOD_IMAGE_ROOT' == foreign(StoreParam.param_code)
        ) """,
        viewonly=True,
        innerjoin=True,
    )
    IMAGE_PATH = property(lambda s: s._compute_image_path())

    def _compute_image_path(self):
        root = self.IMAGE_ROOTS[0].data_value.strip().strip('\\')
        root = root.replace('\\', '/')
        return '%s/%s.JPG' % (root, self.IMGFILENM.strip())



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        String,
                        DateTime,
                        ForeignKey,
                        Numeric,
                        )


class DispensePrice(Carepoint.BASE):
    __tablename__ = 'cprxdisp_pricing'
    __dbname__ = 'cph'

    rxdisp_id = Column(
        Integer,
        ForeignKey('cprx_disp.rxdisp_id'),
        primary_key=True,
    )
    pay_type_cn = Column(Integer)
    t_ingr_cost_paid = Column(Numeric)
    t_ingr_cost_sub = Column(Numeric)
    ingr_cost_cn = Column(Integer)
    aac_cost = Column(Numeric)
    uc_cost = Column(Numeric)
    tpty_claim_id = Column(Integer)
    t_disp_fee_sub = Column(Numeric)
    t_disp_fee_paid = Column(Numeric)
    t_tax_sub = Column(Numeric)
    t_tax_paid = Column(Numeric)
    t_other_amt_sub = Column(Numeric)
    t_other_amt_paid = Column(Numeric)
    t_patient_pay_sub = Column(Numeric)
    t_patient_pay_paid = Column(Numeric)
    app_deduct = Column(Numeric)
    ex_max_ben = Column(Numeric)
    prod_sel_amt = Column(Numeric)
    t_copay_sub = Column(Numeric)
    t_copay_paid = Column(Numeric)
    profit_made = Column(Numeric)
    t_cost_sub = Column(Numeric)
    t_cost_paid = Column(Numeric)
    coupon_no = Column(Integer)
    coupon_type = Column(Integer)
    coupon_amt = Column(Numeric)
    discount = Column(Numeric)
    prior_auth_no = Column(String)
    tpty_auth_no = Column(String)
    t_price_sub = Column(Numeric)
    t_price_paid = Column(Numeric)
    status_cn = Column(Integer)
    primary_pay_date = Column(DateTime)
    second_pay_type_cn = Column(Integer)
    second_amt_paid = Column(Numeric)
    second_auth_no = Column(String)
    disc_grp_name = Column(String)
    primary_amt_paid = Column(Numeric)
    secondary_prior_auth = Column(String)
    other_payor_id_str = Column(String)
    remaining_benefit = Column(Numeric)
    remaining_deductible = Column(Numeric)
    prior_auth_type_cn = Column(Integer)
    prior_auth_type_cn_2 = Column(Integer)
    incentive_fee_sub = Column(Numeric)
    incentive_fee_paid = Column(Numeric)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        String,
                        DateTime,
                        ForeignKey,
                        Numeric,
                        Float,
                        Text,
                        )


class Item(Carepoint.BASE):
    __tablename__ = 'item'
    __dbname__ = 'cph'

    ITEMMSTR = Column(String)
    DESCR = Column(String)
    TAXABLE = Column(String)
    CLASS = Column(String)
    UOFMSALES = Column(String)
    UOFMORDERS = Column(String)
    FACTOR = Column(Float)
    ONHAND = Column(Numeric)
    ONORDER = Column(Numeric)
    VENDOR = Column(String)
    COST = Column(Numeric)
    INQTY = Column(Numeric)
    INDATE = Column(DateTime)
    OUTQTY = Column(Numeric)
    OUTDATE = Column(DateTime)
    ADJQTY = Column(Numeric)
    ADJDATE = Column(DateTime)
    MTD_ISSUE = Column(Numeric)
    MTD_RCPTS = Column(Numeric)
    MTD_ADJ = Column(Numeric)
    COMMENTS = Column(Text)
    IMIN = Column(Numeric)
    IMAX = Column(Numeric)
    UPCCODE = Column(String)
    POSLOC = Column(String)
    VENDITEMNO = Column(String)
    EXCLUDE = Column(String)
    NDC = Column(String)
    SKU = Column(String)
    AUTO_ORD = Column(Integer)
    FIXED_QTY = Column(Numeric)
    ACTIVE_YN = Column(Integer)
    AVG_UNIT_COST = Column(Numeric)
    location = Column(String)
    item_id = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    store_id = Column(
        Integer,
        ForeignKey('csstore.store_id'),
    )
    chemical_id = Column(Integer)
    allocated = Column(Numeric)
    machine_id = Column(Integer)
    special_pkg_ind_cn = Column(Integer)
    refrig_cn = Column(Integer)
    order_multiples_of = Column(Integer)
    add_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    add_date = Column(DateTime)
    chg_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    chg_date = Column(DateTime)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy.types import Enum
from sqlalchemy import (Column,
                        Integer,
                        String
                        )

from carepoint.models.state import EnumOrderState


class OrderStatus(Carepoint.BASE):
    __tablename__ = 'CsOmStatus'
    __dbname__ = 'cph'

    OmStatus = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    state_cn = Column(
        Integer,
    )
    descr = Column(String)

    @property
    def state(self):
        return EnumOrderState(self.state_cn)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        String,
                        )


class PathologyCodeType(Carepoint.BASE):
    __tablename__ = 'fdrfmlitd_10'
    __dbname__ = 'cph'

    icd_cd_type = Column(
        String,
        primary_key=True,
    )
    icd_cd_type_desc = Column(
        String,
    )



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        String,
                        Boolean,
                        )


class FdbRoute(Carepoint.BASE):
    __tablename__ = 'fdrrouted'
    __dbname__ = 'cph'

    gcrt = Column(String, primary_key=True)
    rt = Column(String)
    gcrt2 = Column(String)
    gcrt_desc = Column(String)
    systemic = Column(String)
    update_yn = Column(Boolean)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        String,
                        DateTime,
                        ForeignKey,
                        Numeric,
                        Text,
                        SmallInteger)


class Dispense(Carepoint.BASE):
    __tablename__ = 'cprx_disp'
    __dbname__ = 'cph'

    rxdisp_id = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    rx_id = Column(
        Integer,
        ForeignKey('cprx.rx_id'),
    )
    store_id = Column(
        Integer,
        ForeignKey('csstore.store_id'),
    )
    disp_ndc = Column(String)
    disp_drug_name = Column(String)
    prod_expire_date = Column(DateTime)
    mfg = Column(String)
    orig_mfg = Column(String)
    pkg_size = Column(Numeric)
    fill_no = Column(Integer)
    dispense_date = Column(DateTime)
    dispense_qty = Column(Numeric)
    disp_days_supply = Column(Numeric)
    sig_text = Column(String)
    sched_of_admin_cn = Column(Integer)
    freq_of_admin = Column(Numeric)
    units_per_dose = Column(Numeric)
    lot_number = Column(String)
    disp_awp = Column(Numeric)
    disp_aac = Column(Numeric)
    disp_mac = Column(Numeric)
    disp_ful = Column(Numeric)
    disp_udef = Column(Numeric)
    tech_initials = Column(String)
    rph_initials = Column(String)
    cnsl_initials = Column(String)
    icd9 = Column(String)
    daw_disp_cn = Column(Integer)
    level_of_service = Column(Integer)
    cmt = Column(Text)
    status_cn = Column(Integer)
    alt_pick_up_id = Column(Integer)
    alt_pick_up_cn = Column(Integer)
    clarification_fill = Column(Integer)
    trip_no = Column(Integer)
    gpi_disp = Column(Integer)
    label_3pty_yn = Column(Integer)
    reject_3pty_yn = Column(Integer)
    pay_type_cn = Column(Integer)
    price_differs_yn = Column(Integer)
    pat_loc_cn = Column(Integer)
    billing_units = Column(String)
    price_table_id = Column(Integer)
    app_flags = Column(Integer)
    timestmp = Column(DateTime)
    price_meth_cn = Column(Integer)
    uu = Column(String)
    p_tbl_override_yn = Column(Integer)
    label_id = Column(Integer)
    billing_hold = Column(Integer)
    post_bal_yn = Column(Integer)
    inv_no = Column(String)
    disp_qty_delta = Column(Numeric)
    brand_med_nec_yn = Column(Integer)
    Other_coverage_cd = Column(Integer)
    epsdt_yn = Column(Integer)
    exempt_cd = Column(Integer)
    num_labels = Column(Integer)
    location = Column(String)
    qty_override = Column(Numeric)
    dur_summary = Column(Text)
    cov_overrides = Column(Integer)
    use_secondary_ins_yn = Column(Integer)
    hp_blnRxExtr = Column(SmallInteger)
    item_id = Column(Integer)
    ud_override = Column(String)
    disp_status_cn = Column(Integer)
    order_id = Column(Integer)
    sig_id = Column(Integer)
    counsel_yn = Column(Integer)
    processing_date = Column(DateTime)
    hcpcs_mod_cn = Column(Integer)
    acct_id = Column(Integer)
    track_pat_resp_yn = Column(Integer)
    track_org_resp_yn = Column(Integer)
    label_fac_id = Column(Integer)
    extern_process_cn = Column(Integer)
    price_ovr_user_id = Column(Integer)
    price_ovr_reason_cn = Column(Integer)
    wf_status_cn = Column(Integer)
    verify_user_id = Column(Integer)
    verify_timestamp = Column(DateTime)
    ar_hold = Column(Integer)
    disp_type_cn = Column(Integer)
    own_use_pricing_yn = Column(Integer)
    pos_processed_yn = Column(Integer)
    prescriber_cn = Column(Integer)
    visit_nbr = Column(String)
    disp_udef = Column(Integer)
    SecondaryICD9 = Column(String)
    uandc_pricing_used_yn = Column(Integer)
    special_pkg_ind_cn = Column(Integer)
    delay_reason_cn = Column(Integer)
    place_of_service_cn = Column(Integer)
    pat_residence_cn = Column(Integer)
    compound_type_cn = Column(Integer)
    pharmacy_service_type_cn = Column(Integer)
    admin_start_date = Column(DateTime)
    ClariFill_2 = Column(Integer)
    ClariFill_3 = Column(Integer)
    other_coverage_cd_2 = Column(Integer)
    pat_assign_ind_yn = Column(Integer)
    prov_assign_ind_yn = Column(Integer)
    route_of_admin_ovr = Column(String)
    csr_pickup_id = Column(Integer)
    icd10 = Column(String)
    secondaryicd10 = Column(String)
    add_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    add_date = Column(DateTime)
    chg_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    chg_date = Column(DateTime)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import unittest
from sqlalchemy.schema import Table
from carepoint.tests.db.db import DatabaseTest
from carepoint.models.cph.user import EnumUserType, User


class TestModelsCphUser(DatabaseTest):

    def new_record(self, type_cd='U'):
        self.type_cd = type_cd
        obj = User()
        obj.user_type_cd = type_cd
        return obj

    def test_table_initialization(self, ):
        self.assertIsInstance(User.__table__, Table)

    def test_user_type(self):
        """ It should return proper Enum for user type cd """
        obj = self.new_record()
        self.assertEqual(
            obj.user_type,
            EnumUserType(self.type_cd),
        )


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import unittest
from sqlalchemy.schema import Table
from carepoint.tests.db.db import DatabaseTest
from carepoint.models.phone_mixin import PhoneMixin, EnumPhoneType


class TestPhoneMixin(DatabaseTest):

    def setUp(self):
        super(TestPhoneMixin, self).setUp()
        self.type_cn = 2
        self.obj = PhoneMixin()
        self.obj.phone_type_cn = self.type_cn

    def test_addr_type(self):
        """ It should return proper Enum for type cn """
        self.assertEqual(
            EnumPhoneType(self.type_cn),
            self.obj.phone_type,
        )


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from mixer.backend.sqlalchemy import mixer
from datetime import datetime


dt_now = datetime.now()
__model__ = 'carepoint.models.cph.user.User'


user_default = mixer.blend(
    __model__,
    user_id=1,
    user_type_cd='UserType',
    login_name='LoginName',
    lname='Lname',
    fname='Fname',
    mname='Mname',
    title_lu='TitleLu',
    suffix_lu='SuffixLu',
    degree_lu='Degree',
    initials='Initials',
    job_title_lu='JobTitle',
    anote='Anote',
    employee_no='EmployeeNo',
    ssn='Social',
    last_login_date=dt_now,
    email='Email',
    db_password='DbPassword',
    es_password='EsPassword',
    cmt='Cmt',
    system_yn=1,
    license_no='LicenseNo',
    license_state_cd='LicenseStateCd',
    status_cn=1,
    app_flags=1,
    timestmp=dt_now,
    must_change_password_yn=1,
    password_date=dt_now,
    add_user_id=1,
    add_date=dt_now,
    chg_user_id=1,
    chg_date=dt_now,
)


def user_rnd(cnt):
    return mixer.cycle(cnt).blend(__model__)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from mixer.backend.sqlalchemy import mixer
from datetime import datetime


dt_now = datetime.now()
__model__ = 'carepoint.models.cph.store_param.StoreParam'


store_param_default = mixer.blend(
    __model__,
    store_id=1,
    param_code='SC_MOD_RD_PROD_IMAGE_DIR',
    descr='SC_MOD_RD_PROD_IMAGE_DIR',
    data_value='\\GRXServer\images',
    app_flags=0,
    timestmp=dt_now
)


def store_param_rnd(cnt):
    return mixer.cycle(cnt).blend(__model__)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from mixer.backend.sqlalchemy import mixer
from datetime import datetime


dt_now = datetime.now()
__model__ = 'carepoint.models.cph.fdb_ndc.FdbNdc'


fdb_ndc_default = mixer.blend(
    __model__,
    id=1,
    ncd=2032902,
    lblrid='A00002',
    gcn_seqno=8879,
    ps=100,
    df=1,
    ad='PEDIATRIC',
    ln='V-CILLIN K 250MG TABLET',
    bn='V-CILLIN K',
    pndc=777050324,
    repndc=66591065141,
    ndcfi=1,
    daddnc=19820101,
    dupdc=20001102,
    desi=None,  # All Blank in test DB
    desdtec=0,  # All 0 in test DB
    desi2=None,  # All Blank in test DB
    des2dtec=0,  # All 0 in test DB
    dea=0,  # 0,1,2,3,4 (Scheduled Substance)
    cl='F',  # T, F in Test DB - likely Bool conversion
    gpi=2,  # 0,1,2,3
    hosp=1,  # 0,1
    innov=1,  # 0,1
    ipi=0,  # 0,1
    mini=1,  # 0,1
    maint=None,  # 1 or NULL
    obc='AB',
    obsdtec=19971224,
    ppi=None,  # 1 or NULL
    stpk=1,  # 0,1
    repack=0,  # 0,1
    top200=0,  # 0,1
    ud=0,  # 0,1
    csp=1,
    color='WHITE',
    flavor='CHERRY',
    shape='ELLIPTICAL',
    ndl_gdge=0,  # 0
    ndl_lngth=0,  # 0
    syr_cpcty=3,  # 0,1,2,3
    shlf_pck=24,
    shipper=1,
    skey=200476000000000000000000,
    hcfa_fda='AB',
    hcfa_unit='TAB',
    hcfa_ps=1,
    hcfa_appc=19580926,
    hcfa_mrkc=19580926,
    hcfa_trmc=19971001,
    hcfa_typ=2,
    hcfa_desc1=19930101,
    hcfa_desi1=2,
    uu=0,  # 0,1
    pd='BOTTLE',
    ln25='V-CILLIN K  250MG TABLILL',
    ln25i=0,
    gpidc=19940101,
    bbdc=19970108,
    home=0,  # 0,1
    inpcki=0,  # 0,1
    outpcki=0,  # 0,1
    obc_exp='AB',
    ps_equiv=100,
    plblr=0,  # 0,1
    hcpc='J3260',
    top50gen=0,
    obc3='AB',
    gmi=2,  # 0,1,2
    gni=3,  # 0,1,2,3
    gsi=9,
    gti=2,
    ndcgi1=1,
    user_gcdf=None,  # All blank
    user_str=None,  # All blank
    real_product_yn=None,  # All blank
    no_update_yn=None,  # All blank
    no_prc_update_yn=None,  # All blank
    user_product_yn=0,  # 0 & Blank
    cpname_short=None,  # All blank
    status_cn=None,  # All blank
    update_yn=None,  # All blank
    active_yn=0,  # 0,1
    ln60='DARVOCET-N 50 TABLET',
)


def fdb_ndc_rnd(cnt):
    return mixer.cycle(cnt).blend(__model__)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from mixer.backend.sqlalchemy import mixer
from datetime import datetime


dt_now = datetime.now()
__model__ = 'carepoint.models.cph.fdb_gcn_seq.FdbGcnSeq'


fdb_gcn_seq_default = mixer.blend(
    __model__,
    gcn_seqno=1,
    hic3='A1A',
    hicl_seqno=1,
    gcdf='TA',
    gcrt=1,
    str='100 MG',
    gtc=38,
    tc=74,
    dcc=0,
    gcnseq_gi=1,
    gender=0,
    hic3_seqn=139,
    str60='100 mg',
    update_yn=0,
)


def fdb_gcn_seq_rnd(cnt):
    return mixer.cycle(cnt).blend(__model__)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from mixer.backend.sqlalchemy import mixer
from datetime import datetime


dt_now = datetime.now()
__model__ = 'carepoint.models.cph.store.Store'


store_default = mixer.blend(
    __model__,
    store_id=1,
    store_type_cn=1,
    name='Name',
    store_hours='StoreHours',
    store_no='StoreNo',
    fed_tax_id='FedTaxId',
    url='Url',
    email='Email',
    mgr_cont_id=1,
    cont_id=1,
    carepoint_acct_no='CarepointAcctNo',
    cmt='Cmt',
    status_cn=1,
    app_flags=1,
    nabp='Nabp',
    medicaid_no='Medicaid',
    timestmp=dt_now,
    region_id=1,
    NPI='Npi',
    pharmacy_service_type_cn=1,
    web_refill_yn=False,
    add_user_id=1,
    add_date=dt_now,
    chg_user_id=1,
    chg_date=dt_now,
)


def store_rnd(cnt):
    return mixer.cycle(cnt).blend(__model__)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        String,
                        DateTime,
                        Numeric,
                        Text,
                        ForeignKey,
                        )


class Pathology(Carepoint.BASE):
    __tablename__ = 'fdrfmlinm_10'
    __dbname__ = 'cph'

    icd_cd = Column(
        String,
        primary_key=True,
    )
    icd_cd_type = Column(
        String,
        ForeignKey('fdrfmlitd_10.icd_cd_type'),
        primary_key=True,
    )
    icd_desc = Column(Text)
    icd_desc_source_cd = Column(String)
    icd_status_cd = Column(String)
    icd_first_dt = Column(DateTime)
    icd_last_dt = Column(DateTime)
    icd_billable_ind = Column(Numeric(1, 0))



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import unittest
from carepoint.db import Db, Carepoint
from sqlalchemy.orm.session import Session


class DatabaseTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls, ):
        cls.engine = Db(drv=Db.SQLITE)
        cls.connection = cls.engine.connect()
        cls.transaction = cls.connection.begin()
        Carepoint.BASE.metadata.create_all(cls.connection)

    @classmethod
    def tearDownClass(cls, ):
        # cls.transaction.rollback()
        cls.connection.close()
        cls.engine.dispose()

    def setUp(self, ):
        self.__transaction = self.connection.begin_nested()
        self.session = Session(self.connection)

    def tearDown(self, ):
        self.session.close()
        # self.__transaction.rollback()



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from mixer.backend.sqlalchemy import mixer
from datetime import datetime

dt_now = datetime.now()
__model__ = 'carepoint.models.cph.doctor.Doctor'


doctor_default = mixer.blend(
    __model__,
    md_id=1,
    lname='LName',
    lname_sdx='LName',
    fname='FName',
    fname_sdx='FName',
    mname='MName',
    title_lu='TitlaLu',
    suffix_lu='SuffixLu',
    degree_lu='DgreeLu',
    prescriber_type_cn=1,
    email='Email',
    url='Url',
    pref_meth_contact_cn=1,
    best_cont_time='BestContTime',
    specialty_cd='SpecialtyCd',
    dea_no='DeaNo',
    dea_suffix='DeaSuffix',
    spin_no='SpinNo',
    upin_no='UpinNo',
    other_id='OtherId',
    fed_tax_id='FedTaxId',
    hin_id='HiId',
    champus_id='ChampusId',
    state_lic_id='StateLicId',
    ncpdp_id='NcpdpId',
    medicaid_id='MedicaidId',
    medicare_id='MedicareId',
    blue_shield_id='BlueShieldId',
    blue_cross_id='BlueCrossId',
    npi_id='NpiId',
    practice_org_id=1,
    cmt='cmt',
    status_cn=1,
    conv_code='ConvCode',
    app_flags='AppFlags',
    timestmp=dt_now,
    state_issued_id='StateIssuedId',
    sname='Sname',
    medicaid_restricted_yn=False,
    fac_id=1,
    C2Restricted_yn=False,
    co_md_id=1,
    do_not_refer=1,
    deceased_yn=False,
    C0restricted_yn=False,
    C3restricted_yn=False,
    C4restricted_yn=False,
    C5restricted_yn=False,
    study_drugs_yn=False,
    tracking_cn=1,
    CTP_no='CTPno',
    add_user_id=1,
    add_date=False,
    chg_user_id=1,
    chg_date=False,
)


def doctor_rnd(cnt):
    return
    mixer.cycle(cnt).blend(__model__)



# -*- coding: utf-8 -*-
# Copyright 2016 LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import enum

from sqlalchemy import (Column,
                        Integer,
                        DateTime,
                        ForeignKey,
                        )
from sqlalchemy.types import Enum
from sqlalchemy.ext.declarative import declared_attr


class EnumAddressType(enum.Enum):
    """ It provides PEP-0435 compliant Carepoint Address Type Enumerable """

    business = 1
    home = 2
    school = 3
    vacation = 4
    other = 5
    billing = 6
    shipping = 7


class AddressMixin(object):
    """ This is a mixin for Address Many2Many bindings """

    priority = Column(Integer)
    addr_type_cn = Column(Integer)
    app_flags = Column(Integer)
    timestmp = Column(DateTime)

    @declared_attr
    def addr_id(cls):
        return Column(
            Integer,
            ForeignKey('csaddr.addr_id'),
            primary_key=True,
        )

    @property
    def addr_type(self):
        return EnumAddressType(self.addr_type_cn)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from mixer.backend.sqlalchemy import mixer
from datetime import datetime


dt_now = datetime.now()
__model__ = 'carepoint.models.cph.fdb_gcn.FdbGcn'


fdb_gcn_default = mixer.blend(
    __model__,
    gcn_seqno=1,
    gcn=11,
    update_yn=False,
)


def fdb_gcn_rnd(cnt):
    return mixer.cycle(cnt).blend(__model__)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        String,
                        DateTime,
                        Boolean,
                        ForeignKey,
                        Text,
                        SmallInteger)


class Patient(Carepoint.BASE):
    __dbname__ = 'cph'
    __tablename__ = 'cppat'

    pat_id = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    cmt_id = Column(Integer)
    pat_status_cn = Column(Integer)
    pat_type_cn = Column(Integer)
    nh_pat_id = Column(String)
    chart_id = Column(String)
    lname = Column(String)
    lname_sdx = Column(String)
    fname = Column(String)
    fname_sdx = Column(String)
    mname = Column(String)
    title_lu = Column(String)
    suffix_lu = Column(String)
    alias = Column(String)
    mmname = Column(String)
    alt1_id = Column(String)
    pref_meth_cont_cn = Column(String)
    best_cont_time = Column(String)
    ssn = Column(String)
    dln = Column(String)
    dln_state_cd = Column(String)
    email = Column(String)
    gender_cd = Column(String)
    birth_date = Column(DateTime)
    death_date = Column(DateTime)
    no_safety_caps_yn = Column(Boolean)
    generics_yn = Column(Boolean)
    label_style_cn = Column(Integer)
    primary_md_id = Column(
        Integer,
        ForeignKey('cpmd.md_id')
    )
    secondary_md_id = Column(
        Integer,
        ForeignKey('cpmd.md_id')
    )
    edu_level_cn = Column(Integer)
    ethnicity_cd = Column(String)
    marital_status_cd = Column(String)
    religion_cn = Column(Integer)
    name_spouse = Column(String)
    primary_lang_cd = Column(String)
    rec_release_status_cn = Column(Integer)
    rec_release_date = Column(DateTime)
    hh_relation_cn = Column(Integer)
    hh_pat_id = Column(
        Integer,
        ForeignKey('cppat.pat_id'),
    )
    fam_relation_cn = Column(String)
    fam_pat_id = Column(
        Integer,
        ForeignKey('cppat.pat_id'),
    )
    primary_store_id = Column(
        Integer,
        ForeignKey('csstore.store_id'),
    )
    bad_check_yn = Column(Boolean)
    price_formula_id = Column(Integer)
    cmt = Column(Text)
    status_cn = Column(Integer)
    facility_pat_yn = Column(Boolean)
    conv_code = Column(String)
    app_flags = Column(Integer)
    timestmp = Column(DateTime)
    comp_cn = Column(Integer)
    hp_blnAdmExt = Column(SmallInteger)
    hp_ExtAtt = Column(SmallInteger)
    user_def_1 = Column(String)
    user_def_2 = Column(String)
    user_def_3 = Column(String)
    user_def_4 = Column(String)
    sc_pat_id = Column(
        Integer,
        ForeignKey('cppat.pat_id'),
    )
    resp_party_id = Column(Integer)
    auto_refill_cn = Column(Integer)
    fill_yn = Column(Boolean)
    fill_stop_date = Column(DateTime)
    fill_resume_date = Column(DateTime)
    fill_stop_reason_cn = Column(Integer)
    fill_stop_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    registration_date = Column(DateTime)
    anonymous_yn = Column(Boolean)
    market_yn = Column(Boolean)
    representative = Column(String)
    discharge_date = Column(DateTime)
    do_not_resuscitate_yn = Column(Boolean)
    discharge_exp_date = Column(DateTime)
    pat_loc_cn = Column(Integer)
    rx_priority_default_cn = Column(Integer)
    ship_cn = Column(Integer)
    residence_cn = Column(Integer)
    add_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    add_date = Column(DateTime)
    chg_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    chg_date = Column(DateTime)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        Boolean,
                        String,
                        ForeignKey,
                        DateTime,
                        )


class FdbNdcCsExt(Carepoint.BASE):
    __tablename__ = 'csext_fdrndc'
    __dbname__ = 'cph'

    ndc = Column(String, primary_key=True)
    short_name = Column(String)
    lot_no = Column(String)
    orig_mfg = Column(String)
    pref_gen_yn = Column(Boolean)
    active_yn = Column(Boolean)
    drug_expire_days = Column(String)
    formulary_yn = Column(Boolean)
    compound_yn = Column(Boolean)
    sup_upd_gen_yn = Column(Boolean)
    sup_upd_phys_yn = Column(Boolean)
    sup_upd_clin_yn = Column(Boolean)
    sup_upd_fin_yn = Column(Boolean)
    sup_upd_med_yn = Column(Boolean)
    dn_form = Column(String)
    dn_str = Column(String)
    dn_route = Column(String)
    rx_only_yn = Column(Boolean)
    manual_yn = Column(Boolean)
    brand_ndc = Column(String)
    add_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    add_date = Column(DateTime)
    chg_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    app_flags = Column(String)
    timestmp = Column(DateTime)
    comp_yn = Column(Boolean)
    dea = Column(String)
    dea_chg_user = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    dea_chg_date = Column(DateTime)
    ln = Column(String)
    ln_chg_user = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    ln_chg_date = Column(DateTime)
    fdb_chg_date = Column(DateTime)
    ud_svc_code = Column(String)
    gpi = Column(String)
    gpi_chg_user = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    gpi_chg_date = Column(DateTime)
    bill_increment = Column(Integer)
    formula_id = Column(Integer)
    alt_iptside1 = Column(String)
    alt_iptside2 = Column(String)
    dose_multiplier = Column(String)
    default_daw_override = Column(String)
    manual_price_yn = Column(Boolean)
    compound_type_cn = Column(Integer)
    refrig_cn = Column(Integer)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from mixer.backend.sqlalchemy import mixer


__model__ = 'carepoint.models.cph.fdb_route.FdbRoute'


fdb_route_default = mixer.blend(
    __model__,
    gcrt='K',
    rt='INTRAARTIC',
    gcrt2='UI',
    gcrt_desc='INTRAARTICULAR',
    systemic='S',
    update_yn=False,
)


def fdb_route_rnd(cnt):
    return mixer.cycle(cnt).blend(__model__)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from mixer.backend.sqlalchemy import mixer
from datetime import datetime


dt_now = datetime.now()
__model__ = 'carepoint.models.cph.fdb_ndc_cs_ext.FdbNdcCsExt'


fdb_ndc_cs_ext_default = mixer.blend(
    __model__,
    ndc='0002032902',
    short_name='An item',
    lot_no='A lot',
    orig_mfg='Orig Mfg',
    pref_gen_yn=True,
    active_yn=True,
    drug_expire_days='Drug Expire Days',
    formulary_yn=True,
    compound_yn=True,
    sup_upd_gen_yn=True,
    sup_upd_phys_yn=True,
    sup_upd_clin_yn=True,
    sup_upd_fin_yn=True,
    sup_upd_med_yn=True,
    dn_form='TABLET',
    dn_str='250 MG',
    dn_route='ORAL',
    rx_only_yn=True,
    manual_yn=True,
    brand_ndc='0002032902',
    add_user_id=1,
    add_date=dt_now,
    chg_user_id=1,
    app_flags='',
    timestmp=dt_now,
    comp_yn=True,
    dea='Dea Num',
    dea_chg_user=1,
    dea_chg_date=dt_now,
    ln='Ln ',
    ln_chg_user=1,
    ln_chg_date=dt_now,
    fdb_chg_date=dt_now,
    ud_svc_code='Ud Svc Code',
    gpi='Gpi',
    gpi_chg_user=1,
    gpi_chg_date=dt_now,
    bill_increment=1,
    formula_id=1,
    alt_iptside1='Alt IptSide1',
    alt_iptside2='Alt IptSide2',
    dose_multiplier='Dose Multiplier',
    default_daw_override='Daw Override',
    manual_price_yn=True,
    compound_type_cn=1,
    refrig_cn=1,
)


def fdb_ndc_cs_ext_rnd(cnt):
    return mixer.cycle(cnt).blend(__model__)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        String,
                        DateTime,
                        Boolean,
                        ForeignKey,
                        Text,
                        )


class Store(Carepoint.BASE):
    __tablename__ = 'csstore'
    __dbname__ = 'cph'

    store_id = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    store_type_cn = Column(Integer)
    name = Column(String)
    store_hours = Column(String)
    store_no = Column(String)
    fed_tax_id = Column(String)
    url = Column(String)
    email = Column(String)
    mgr_cont_id = Column(Integer)
    cont_id = Column(Integer)
    carepoint_acct_no = Column(String)
    cmt = Column(Text)
    status_cn = Column(Integer)
    app_flags = Column(Integer)
    nabp = Column(String)
    medicaid_no = Column(String)
    timestmp = Column(DateTime)
    region_id = Column(Integer)
    NPI = Column(String)
    pharmacy_service_type_cn = Column(Integer)
    web_refill_yn = Column(Boolean)
    add_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    add_date = Column(DateTime)
    chg_user_id = Column(
        Integer,
        ForeignKey('csuser.user_id'),
    )
    chg_date = Column(DateTime)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        String,
                        Boolean,
                        ForeignKey,
                        )


class FdbGcnSeq(Carepoint.BASE):
    __tablename__ = 'fdrgcnseq'
    __dbname__ = 'cph'

    gcn_seqno = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    hic3 = Column(String)
    hicl_seqno = Column(Integer)
    gcdf = Column(
        String,
        ForeignKey('fdrdosed.gcdf'),
    )
    gcrt = Column(
        String,
        ForeignKey('fdrrouted.gcrt'),
    )
    str = Column(String)
    gtc = Column(Integer)
    tc = Column(Integer)
    dcc = Column(Integer)
    gcnseq_gi = Column(Integer)
    gender = Column(Integer)
    hic3_seqn = Column(Integer)
    str60 = Column(String)
    update_yn = Column(Boolean)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        String,
                        DateTime,
                        ForeignKey,
                        )


class StoreParam(Carepoint.BASE):
    __tablename__ = 'csstore_param'
    __dbname__ = 'cph'

    param_code = Column(String, primary_key=True)
    store_id = Column(
        Integer,
        ForeignKey('csstore.store_id'),
    )
    descr = Column(String)
    data_value = Column(String)
    app_flags = Column(Integer)
    timestmp = Column(DateTime)



# -*- coding: utf-8 -*-
# Copyright 2016 LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import enum


class EnumOrderState(enum.Enum):
    """ It provides PEP-0435 compliant Carepoint State Enumerable """

    entered = 10
    verified = 20
    adjudicated = 30
    processed = 35
    approved = 40
    shipped = 50
    returned = 60



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from mixer.backend.sqlalchemy import mixer


__model__ = 'carepoint.models.cph.fdb_form.FdbForm'


fdb_form_default = mixer.blend(
    __model__,
    gcdf='0A',
    dose='UNIDENT',
    gcdf_desc='UNIDENTIFIED',
    update_yn=False,
)


def fdb_form_rnd(cnt):
    return mixer.cycle(cnt).blend(__model__)



# -*- coding: utf-8 -*-
# Copyright 2015-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from carepoint import Carepoint
from sqlalchemy import (Column,
                        Integer,
                        Boolean,
                        String,
                        )


class FdbGcn(Carepoint.BASE):
    __tablename__ = 'fdrgcn'
    __dbname__ = 'cph'

    gcn_seqno = Column(
        Integer, 
        primary_key=True,
        autoincrement=False,
    )
    gcn = Column(String)
    update_yn = Column(Boolean)
